@@ -87,6 +87,7 @@ public:
 	void requestDialogEntry(
 		not_null<History*> history,
 		Fn<void()> callback = nullptr);
+	void requestDialogEntries(std::vector<not_null<History*>> histories);
 	//void applyFeedSources(const MTPDchannels_feedSources &data); // #feed
 	//void setFeedChannels(
 	//	not_null<Data::Feed*> feed,
@@ -124,6 +125,12 @@ public:
 	void requestTermsUpdate();
 	void acceptTerms(bytes::const_span termsId);
 
+	void checkChatInvite(
+		const QString &hash,
+		FnMut<void(const MTPChatInvite &)> done,
+		FnMut<void(const RPCError &)> fail);
+	void importChatInvite(const QString &hash);
+
 	void requestChannelMembersForAdd(
 		not_null<ChannelData*> channel,
 		Fn<void(const MTPchannels_ChannelParticipants&)> callback);
@@ -262,6 +269,7 @@ public:
 		WebPageId webPageId = 0;
 		bool clearDraft = false;
 		bool generateLocal = true;
+		bool handleSupportSwitch = false;
 	};
 	rpl::producer<SendOptions> sendActions() const {
 		return _sendActions.events();
@@ -318,8 +326,10 @@ public:
 		MsgId replyTo = 0;
 		WebPageId webPageId = 0;
 		bool clearDraft = true;
+		bool handleSupportSwitch = false;
 	};
 	void sendMessage(MessageToSend &&message);
+	void sendBotStart(not_null<UserData*> bot);
 	void sendInlineResult(
 		not_null<UserData*> bot,
 		not_null<InlineBots::Result*> data,
@@ -347,6 +357,7 @@ public:
 			LastSeen,
 			Calls,
 			Invites,
+			CallsPeer2Peer,
 		};
 		enum class Option {
 			Everyone,
@@ -704,6 +715,10 @@ private:
 	TimeMs _termsUpdateSendAt = 0;
 	mtpRequestId _termsUpdateRequestId = 0;
 
+	mtpRequestId _checkInviteRequestId = 0;
+	FnMut<void(const MTPChatInvite &result)> _checkInviteDone;
+	FnMut<void(const RPCError &error)> _checkInviteFail;
+
 	std::vector<FnMut<void(const MTPUser &)>> _supportContactCallbacks;
 
 	base::flat_map<FullMsgId, not_null<PeerData*>> _peerPhotoUploads;
