@@ -7,10 +7,6 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "app.h"
 
-#ifdef OS_MAC_OLD
-#include <libexif/exif-data.h>
-#endif // OS_MAC_OLD
-
 #include "styles/style_overview.h"
 #include "styles/style_mediaview.h"
 #include "styles/style_chat_helpers.h"
@@ -27,6 +23,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "history/history_item_components.h"
 #include "history/view/history_view_service_message.h"
 #include "media/media_audio.h"
+#include "ui/image/image.h"
 #include "inline_bots/inline_bot_layout_item.h"
 #include "messenger.h"
 #include "application.h"
@@ -46,14 +43,15 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "window/notifications_manager.h"
 #include "platform/platform_notifications_manager.h"
 
+#ifdef OS_MAC_OLD
+#include <libexif/exif-data.h>
+#endif // OS_MAC_OLD
+
 namespace {
 	App::LaunchState _launchState = App::Launched;
 
 	std::unordered_map<PeerId, std::unique_ptr<PeerData>> peersData;
 
-	using LocationsData = QHash<LocationCoords, LocationData*>;
-	LocationsData locationsData;
-
 	using DependentItemsSet = OrderedSet<HistoryItem*>;
 	using DependentItems = QMap<HistoryItem*, DependentItemsSet>;
 	DependentItems dependentItems;
@@ -77,7 +75,6 @@ namespace {
 		*pressedLinkItem = nullptr,
 		*mousedItem = nullptr;
 
-	QPixmap *emoji = nullptr, *emojiLarge = nullptr;
 	style::font monofont;
 
 	struct CornersPixmaps {
@@ -88,10 +85,6 @@ namespace {
 	CornersMap cornersMap;
 	QImage cornersMaskLarge[4], cornersMaskSmall[4];
 
-	using EmojiImagesMap = QMap<int, QPixmap>;
-	EmojiImagesMap MainEmojiMap;
-	QMap<int, EmojiImagesMap> OtherEmojiMap;
-
 	int32 serviceImageCacheSize = 0;
 
 } // namespace
@@ -669,7 +662,7 @@ namespace App {
 					bool found = !h || !h->lastKeyboardFrom;
 					auto botStatus = -1;
 					for (auto i = chat->participants.begin(); i != chat->participants.end();) {
-						auto [user, version] = *i;
+						const auto [user, version] = *i;
 						if (version < pversion) {
 							i = chat->participants.erase(i);
 						} else {
@@ -768,7 +761,7 @@ namespace App {
 					}
 					if (chat->botStatus > 0 && user->botInfo) {
 						int32 botStatus = -1;
-						for (auto [participant, v] : chat->participants) {
+						for (const auto [participant, v] : chat->participants) {
 							if (participant->botInfo) {
 								if (true || botStatus > 0/* || !participant->botInfo->readsAllHistory*/) {
 									botStatus = 2;
@@ -957,7 +950,7 @@ namespace App {
 			auto &d = size.c_photoSize();
 			if (d.vlocation.type() == mtpc_fileLocation) {
 				auto &l = d.vlocation.c_fileLocation();
-				return ImagePtr(
+				return Images::Create(
 					StorageImageLocation(
 						d.vw.v,
 						d.vh.v,
@@ -974,7 +967,7 @@ namespace App {
 			if (d.vlocation.type() == mtpc_fileLocation) {
 				auto &l = d.vlocation.c_fileLocation();
 				auto bytes = qba(d.vbytes);
-				return ImagePtr(
+				return Images::Create(
 					StorageImageLocation(
 						d.vw.v,
 						d.vh.v,
@@ -986,7 +979,7 @@ namespace App {
 					bytes);
 			} else if (d.vlocation.type() == mtpc_fileLocationUnavailable) {
 				auto bytes = qba(d.vbytes);
-				return ImagePtr(
+				return Images::Create(
 					StorageImageLocation(
 						d.vw.v,
 						d.vh.v,
@@ -1141,15 +1134,24 @@ namespace App {
 		}
 	}
 
-	void enumerateChatsChannels(
-			Fn<void(not_null<PeerData*>)> action) {
+	void enumerateGroups(Fn<void(not_null<PeerData*>)> action) {
 		for (const auto &[peerId, peer] : peersData) {
-			if (!peer->isUser()) {
+			if (peer->isChat() || peer->isMegagroup()) {
 				action(peer.get());
 			}
 		}
 	}
 
+	void enumerateChannels(Fn<void(not_null<ChannelData*>)> action) {
+		for (const auto &[peerId, peer] : peersData) {
+			if (const auto channel = peer->asChannel()) {
+				if (!channel->isMegagroup()) {
+					action(channel);
+				}
+			}
+		}
+	}
+
 	PeerData *peerByName(const QString &username) {
 		const auto uname = username.trimmed();
 		for (const auto &[peerId, peer] : peersData) {
@@ -1160,20 +1162,6 @@ namespace App {
 		return nullptr;
 	}
 
-	LocationData *location(const LocationCoords &coords) {
-		auto i = locationsData.constFind(coords);
-		if (i == locationsData.cend()) {
-			i = locationsData.insert(coords, new LocationData(coords));
-		}
-		return i.value();
-	}
-
-	void forgetMedia() {
-		for_const (auto location, ::locationsData) {
-			location->thumb->forget();
-		}
-	}
-
 	QString peerName(const PeerData *peer, bool forDialogs) {
 		return peer ? ((forDialogs && peer->isUser() && !peer->asUser()->nameOrPhone.isEmpty()) ? peer->asUser()->nameOrPhone : peer->name) : lang(lng_deleted);
 	}
@@ -1255,7 +1243,6 @@ namespace App {
 
 	void historyClearMsgs() {
 		::dependentItems.clear();
-
 		const auto oldData = base::take(msgsData);
 		const auto oldChannelData = base::take(channelMsgsData);
 		for (const auto item : oldData) {
@@ -1438,15 +1425,6 @@ namespace App {
 			if (family.isEmpty()) family = QFontDatabase::systemFont(QFontDatabase::FixedFont).family();
 			::monofont = style::font(st::normalFont->f.pixelSize(), 0, family);
 		}
-		Ui::Emoji::Init();
-		if (!::emoji) {
-			::emoji = new QPixmap(Ui::Emoji::Filename(Ui::Emoji::Index()));
-            if (cRetina()) ::emoji->setDevicePixelRatio(cRetinaFactor());
-		}
-		if (!::emojiLarge) {
-			::emojiLarge = new QPixmap(Ui::Emoji::Filename(Ui::Emoji::Index() + 1));
-			if (cRetina()) ::emojiLarge->setDevicePixelRatio(cRetinaFactor());
-		}
 
 		createCorners();
 
@@ -1482,26 +1460,16 @@ namespace App {
 
 		histories().clear();
 
-		clearStorageImages();
+		Images::ClearRemote();
 		cSetServerBackgrounds(WallPapers());
-
-		serviceImageCacheSize = imageCacheSize();
 	}
 
 	void deinitMedia() {
-		delete ::emoji;
-		::emoji = nullptr;
-		delete ::emojiLarge;
-		::emojiLarge = nullptr;
-
 		clearCorners();
 
-		MainEmojiMap.clear();
-		OtherEmojiMap.clear();
-
 		Data::clearGlobalStructures();
 
-		clearAllImages();
+		Images::ClearAll();
 	}
 
 	void hoveredItem(HistoryView::Element *item) {
@@ -1556,44 +1524,6 @@ namespace App {
 		return ::monofont;
 	}
 
-	const QPixmap &emoji() {
-		return *::emoji;
-	}
-
-	const QPixmap &emojiLarge() {
-		return *::emojiLarge;
-	}
-
-	const QPixmap &emojiSingle(EmojiPtr emoji, int32 fontHeight) {
-		auto &map = (fontHeight == st::msgFont->height) ? MainEmojiMap : OtherEmojiMap[fontHeight];
-		auto i = map.constFind(emoji->index());
-		if (i == map.cend()) {
-			auto image = QImage(Ui::Emoji::Size() + st::emojiPadding * cIntRetinaFactor() * 2, fontHeight * cIntRetinaFactor(), QImage::Format_ARGB32_Premultiplied);
-            if (cRetina()) image.setDevicePixelRatio(cRetinaFactor());
-			image.fill(Qt::transparent);
-			{
-				QPainter p(&image);
-				emojiDraw(p, emoji, st::emojiPadding * cIntRetinaFactor(), (fontHeight * cIntRetinaFactor() - Ui::Emoji::Size()) / 2);
-			}
-			i = map.insert(emoji->index(), App::pixmapFromImageInPlace(std::move(image)));
-		}
-		return i.value();
-	}
-
-	void checkImageCacheSize() {
-		int64 nowImageCacheSize = imageCacheSize();
-		if (nowImageCacheSize > serviceImageCacheSize + MemoryForImageCache) {
-			App::forgetMedia();
-			Auth().data().forgetMedia();
-			serviceImageCacheSize = imageCacheSize();
-		}
-	}
-
-	bool isValidPhone(QString phone) {
-		phone = phone.replace(QRegularExpression(qsl("[^\\d]")), QString());
-		return phone.length() >= 8 || phone == qsl("777") || phone == qsl("333") || phone == qsl("111") || (phone.startsWith(qsl("42")) && (phone.length() == 2 || phone.length() == 5 || phone == qsl("4242")));
-	}
-
 	void quit() {
 		if (quitting()) {
 			return;
