@@ -9,13 +9,30 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 
 #include "abstract_box.h"
 
+namespace style {
+struct CalendarSizes;
+} // namespace style
+
 namespace Ui {
 class IconButton;
 } // namespace Ui
 
 class CalendarBox : public BoxContent {
 public:
-	CalendarBox(QWidget*, QDate month, QDate highlighted, Fn<void(QDate date)> callback);
+	CalendarBox(
+		QWidget*,
+		QDate month,
+		QDate highlighted,
+		Fn<void(QDate date)> callback,
+		FnMut<void(not_null<CalendarBox*>)> finalize = nullptr);
+	CalendarBox(
+		QWidget*,
+		QDate month,
+		QDate highlighted,
+		Fn<void(QDate date)> callback,
+		FnMut<void(not_null<CalendarBox*>)> finalize,
+		const style::CalendarSizes &st);
+
 
 	void setMinDate(QDate date);
 	void setMaxDate(QDate date);
@@ -33,6 +50,8 @@ private:
 	bool isPreviousEnabled() const;
 	bool isNextEnabled() const;
 
+	const style::CalendarSizes &_st;
+
 	class Context;
 	std::unique_ptr<Context> _context;
 
@@ -45,5 +64,6 @@ private:
 	object_ptr<Ui::IconButton> _next;
 
 	Fn<void(QDate date)> _callback;
+	FnMut<void(not_null<CalendarBox*>)> _finalize;
 
 };
