@@ -8,7 +8,9 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "boxes/edit_caption_box.h"
 
 #include "ui/widgets/input_fields.h"
+#include "ui/image/image.h"
 #include "ui/text_options.h"
+#include "ui/special_buttons.h"
 #include "media/media_clip_reader.h"
 #include "history/history.h"
 #include "history/history_item.h"
@@ -16,13 +18,18 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_photo.h"
 #include "data/data_document.h"
 #include "lang/lang_keys.h"
+#include "core/event_filter.h"
 #include "chat_helpers/message_field.h"
+#include "chat_helpers/tabbed_panel.h"
+#include "chat_helpers/tabbed_selector.h"
+#include "chat_helpers/emoji_suggestions_widget.h"
 #include "window/window_controller.h"
-#include "mainwidget.h"
 #include "layout.h"
 #include "auth_session.h"
+#include "apiwrap.h"
 #include "styles/style_history.h"
 #include "styles/style_boxes.h"
+#include "styles/style_chat_helpers.h"
 
 EditCaptionBox::EditCaptionBox(
 	QWidget*,
@@ -78,13 +85,13 @@ EditCaptionBox::EditCaptionBox(
 					| Images::Option::RoundedTopRight
 					| Images::Option::RoundedBottomLeft
 					| Images::Option::RoundedBottomRight;
-				_thumb = Images::pixmap(
+				_thumb = App::pixmapFromImageInPlace(Images::prepare(
 					image->pix(_msgId).toImage(),
 					_thumbw * cIntRetinaFactor(),
 					0,
 					options,
 					st::msgFileThumbSize,
-					st::msgFileThumbSize);
+					st::msgFileThumbSize));
 			};
 		}
 
@@ -209,13 +216,31 @@ EditCaptionBox::EditCaptionBox(
 		Ui::InputField::Mode::MultiLine,
 		langFactory(lng_photo_caption),
 		editData);
-	_field->setMaxLength(MaxPhotoCaption);
+	_field->setMaxLength(Global::CaptionLengthMax());
 	_field->setSubmitSettings(Ui::InputField::SubmitSettings::Both);
 	_field->setInstantReplaces(Ui::InstantReplaces::Default());
 	_field->setInstantReplacesEnabled(Global::ReplaceEmojiValue());
 	_field->setMarkdownReplacesEnabled(rpl::single(true));
-	_field->setEditLinkCallback(
-		DefaultEditLinkCallback(_controller, _field));
+	_field->setEditLinkCallback(DefaultEditLinkCallback(_field));
+}
+
+bool EditCaptionBox::emojiFilter(not_null<QEvent*> event) {
+	const auto type = event->type();
+	if (type == QEvent::Move || type == QEvent::Resize) {
+		// updateEmojiPanelGeometry uses not only container geometry, but
+		// also container children geometries that will be updated later.
+		crl::on_main(this, [=] { updateEmojiPanelGeometry(); });
+	}
+	return false;
+}
+
+void EditCaptionBox::updateEmojiPanelGeometry() {
+	const auto parent = _emojiPanel->parentWidget();
+	const auto global = _emojiToggle->mapToGlobal({ 0, 0 });
+	const auto local = parent->mapFromGlobal(global);
+	_emojiPanel->moveBottomRight(
+		local.y(),
+		local.x() + _emojiToggle->width() * 3);
 }
 
 void EditCaptionBox::prepareGifPreview(not_null<DocumentData*> document) {
@@ -261,6 +286,11 @@ void EditCaptionBox::prepare() {
 	connect(_field, &Ui::InputField::submitted, [=] { save(); });
 	connect(_field, &Ui::InputField::cancelled, [=] { closeBox(); });
 	connect(_field, &Ui::InputField::resized, [=] { captionResized(); });
+	Ui::Emoji::SuggestionsController::Init(
+		getDelegate()->outerContainer(),
+		_field);
+
+	setupEmojiPanel();
 
 	auto cursor = _field->textCursor();
 	cursor.movePosition(QTextCursor::End);
@@ -270,9 +300,40 @@ void EditCaptionBox::prepare() {
 void EditCaptionBox::captionResized() {
 	updateBoxSize();
 	resizeEvent(0);
+	updateEmojiPanelGeometry();
 	update();
 }
 
+void EditCaptionBox::setupEmojiPanel() {
+	const auto container = getDelegate()->outerContainer();
+	_emojiPanel = base::make_unique_q<ChatHelpers::TabbedPanel>(
+		container,
+		_controller,
+		object_ptr<ChatHelpers::TabbedSelector>(
+			nullptr,
+			_controller,
+			ChatHelpers::TabbedSelector::Mode::EmojiOnly));
+	_emojiPanel->setDesiredHeightValues(
+		1.,
+		st::emojiPanMinHeight / 2,
+		st::emojiPanMinHeight);
+	_emojiPanel->hide();
+	_emojiPanel->getSelector()->emojiChosen(
+	) | rpl::start_with_next([=](EmojiPtr emoji) {
+		Ui::InsertEmojiAtCursor(_field->textCursor(), emoji);
+	}, lifetime());
+
+	_emojiFilter.reset(Core::InstallEventFilter(
+		container,
+		[=](not_null<QEvent*> event) { return emojiFilter(event); }));
+
+	_emojiToggle.create(this, st::boxAttachEmoji);
+	_emojiToggle->installEventFilter(_emojiPanel);
+	_emojiToggle->addClickHandler([=] {
+		_emojiPanel->toggleAnimated();
+	});
+}
+
 void EditCaptionBox::updateBoxSize() {
 	auto newHeight = st::boxPhotoPadding.top() + st::boxPhotoCaptionSkip + _field->height() + errorTopSkip() + st::normalFont->height;
 	if (_photo || _animated) {
@@ -389,6 +450,11 @@ void EditCaptionBox::resizeEvent(QResizeEvent *e) {
 	BoxContent::resizeEvent(e);
 	_field->resize(st::sendMediaPreviewSize, _field->height());
 	_field->moveToLeft(st::boxPhotoPadding.left(), height() - st::normalFont->height - errorTopSkip() - _field->height());
+	_emojiToggle->moveToLeft(
+		(st::boxPhotoPadding.left()
+			+ st::sendMediaPreviewSize
+			- _emojiToggle->width()),
+		_field->y() + st::boxAttachEmojiTop);
 }
 
 void EditCaptionBox::setInnerFocus() {
@@ -443,9 +509,7 @@ void EditCaptionBox::save() {
 void EditCaptionBox::saveDone(const MTPUpdates &updates) {
 	_saveRequestId = 0;
 	closeBox();
-	if (App::main()) {
-		App::main()->sentUpdatesReceived(updates);
-	}
+	Auth().api().applyUpdates(updates);
 }
 
 bool EditCaptionBox::saveFail(const RPCError &error) {
