@@ -83,6 +83,15 @@ std::unique_ptr<PrivacyExceptionsBoxController::Row> PrivacyExceptionsBoxControl
 
 } // namespace
 
+LangKey EditPrivacyBox::Controller::optionLabelKey(Option option) {
+	switch (option) {
+	case Option::Everyone: return lng_edit_privacy_everyone;
+	case Option::Contacts: return lng_edit_privacy_contacts;
+	case Option::Nobody: return lng_edit_privacy_nobody;
+	}
+	Unexpected("Option value in optionsLabelKey.");
+}
+
 EditPrivacyBox::EditPrivacyBox(
 	QWidget*,
 	std::unique_ptr<Controller> controller,
@@ -182,29 +191,21 @@ bool EditPrivacyBox::showExceptionLink(Exception exception) const {
 	Unexpected("Invalid exception value.");
 }
 
-Ui::Radioenum<EditPrivacyBox::Option> *EditPrivacyBox::AddOption(
+Ui::Radioenum<EditPrivacyBox::Option> *EditPrivacyBox::addOption(
 		not_null<Ui::VerticalLayout*> container,
 		const std::shared_ptr<Ui::RadioenumGroup<Option>> &group,
 		Option option) {
-	const auto label = [&] {
-		switch (option) {
-		case Option::Everyone: return lng_edit_privacy_everyone;
-		case Option::Contacts: return lng_edit_privacy_contacts;
-		case Option::Nobody: return lng_edit_privacy_nobody;
-		}
-		Unexpected("Option value in EditPrivacyBox::AddOption.");
-	}();
 	return container->add(
 		object_ptr<Ui::Radioenum<Option>>(
 			container,
 			group,
 			option,
-			lang(label),
+			lang(_controller->optionLabelKey(option)),
 			st::settingsSendType),
 		st::settingsSendTypePadding);
 }
 
-Ui::FlatLabel *EditPrivacyBox::AddLabel(
+Ui::FlatLabel *EditPrivacyBox::addLabel(
 		not_null<Ui::VerticalLayout*> container,
 		rpl::producer<QString> text) {
 	const auto wrap = container->add(
@@ -237,22 +238,20 @@ void EditPrivacyBox::setupContent() {
 
 	const auto group = std::make_shared<Ui::RadioenumGroup<Option>>(
 		_value.option);
-	const auto toggle = Ui::AttachAsChild(content, rpl::event_stream<>());
+	const auto toggle = Ui::CreateChild<rpl::event_stream<>>(content);
 
 	group->setChangedCallback([=](Option value) {
 		_value.option = value;
 		toggle->fire({});
 	});
 
-	const auto addOption = [&](Option option) {
+	const auto addOptionRow = [&](Option option) {
 		return (_controller->hasOption(option) || (_value.option == option))
-			? AddOption(content, group, option)
+			? addOption(content, group, option)
 			: nullptr;
 	};
 	const auto addExceptionLink = [=](Exception exception) {
-		const auto update = Ui::AttachAsChild(
-			content,
-			rpl::event_stream<>());
+		const auto update = Ui::CreateChild<rpl::event_stream<>>(content);
 		auto label = update->events_starting_with(
 			rpl::empty_value()
 		) | rpl::map([=] {
@@ -286,10 +285,10 @@ void EditPrivacyBox::setupContent() {
 	};
 
 	AddSubsectionTitle(content, _controller->optionsTitleKey());
-	addOption(Option::Everyone);
-	addOption(Option::Contacts);
-	addOption(Option::Nobody);
-	AddLabel(content, _controller->warning());
+	addOptionRow(Option::Everyone);
+	addOptionRow(Option::Contacts);
+	addOptionRow(Option::Nobody);
+	addLabel(content, _controller->warning());
 	AddSkip(content);
 
 	AddDivider(content);
@@ -297,11 +296,9 @@ void EditPrivacyBox::setupContent() {
 	AddSubsectionTitle(content, lng_edit_privacy_exceptions);
 	const auto always = addExceptionLink(Exception::Always);
 	const auto never = addExceptionLink(Exception::Never);
-	AddLabel(content, _controller->exceptionsDescription());
+	addLabel(content, _controller->exceptionsDescription());
 	AddSkip(content);
 
-	const auto saveAdditional = _controller->setupAdditional(content);
-
 	addButton(langFactory(lng_settings_save), [=] {
 		const auto someAreDisallowed = (_value.option != Option::Everyone)
 			|| !_value.never.empty();
@@ -309,9 +306,6 @@ void EditPrivacyBox::setupContent() {
 			Auth().api().savePrivacy(
 				_controller->apiKey(),
 				collectResult());
-			if (saveAdditional) {
-				saveAdditional();
-			}
 			closeBox();
 		}));
 	});
