@@ -42,6 +42,61 @@ base::flat_set<not_null<UserData*>> GetAlreadyInFromPeer(PeerData *peer) {
 	return {};
 }
 
+void ShareBotGame(not_null<UserData*> bot, not_null<PeerData*> chat) {
+	const auto history = App::historyLoaded(chat);
+	const auto randomId = rand_value<uint64>();
+	const auto requestId = MTP::send(
+		MTPmessages_SendMedia(
+			MTP_flags(0),
+			chat->input,
+			MTP_int(0),
+			MTP_inputMediaGame(
+				MTP_inputGameShortName(
+					bot->inputUser,
+					MTP_string(bot->botInfo->shareGameShortName))),
+			MTP_string(""),
+			MTP_long(randomId),
+			MTPnullMarkup,
+			MTPnullEntities),
+		App::main()->rpcDone(&MainWidget::sentUpdatesReceived),
+		App::main()->rpcFail(&MainWidget::sendMessageFail),
+		0,
+		0,
+		history ? history->sendRequestId : 0);
+	if (history) {
+		history->sendRequestId = requestId;
+	}
+	Ui::hideLayer();
+	Ui::showPeerHistory(chat, ShowAtUnreadMsgId);
+}
+
+void AddBotToGroup(not_null<UserData*> bot, not_null<PeerData*> chat) {
+	if (auto &info = bot->botInfo) {
+		if (!info->startGroupToken.isEmpty()) {
+			MTP::send(
+				MTPmessages_StartBot(
+					bot->inputUser,
+					chat->input,
+					MTP_long(rand_value<uint64>()),
+					MTP_string(info->startGroupToken)),
+				App::main()->rpcDone(&MainWidget::sentUpdatesReceived),
+				App::main()->rpcFail(
+					&MainWidget::addParticipantFail,
+					{ bot, chat }));
+		} else {
+			App::main()->addParticipants(
+				chat,
+				{ 1, bot });
+		}
+	} else {
+		App::main()->addParticipants(
+			chat,
+			{ 1, bot });
+	}
+	Ui::hideLayer();
+	Ui::showPeerHistory(chat, ShowAtUnreadMsgId);
+}
+
 } // namespace
 
 // Not used for now.
@@ -72,15 +127,24 @@ base::flat_set<not_null<UserData*>> GetAlreadyInFromPeer(PeerData *peer) {
 //	return mapFromGlobal(QCursor::pos()) - _st.rippleAreaPosition;
 //}
 
-class EditChatAdminsBoxController::LabeledCheckbox : public TWidget, private base::Subscriber {
+class EditChatAdminsBoxController::LabeledCheckbox : public Ui::RpWidget {
 public:
-	LabeledCheckbox(QWidget *parent, const QString &text, bool checked = false, const style::Checkbox &st = st::defaultCheckbox, const style::Check &checkSt = st::defaultCheck);
-
-	base::Observable<bool> checkedChanged;
+	LabeledCheckbox(
+		QWidget *parent,
+		const QString &text,
+		bool checked = false,
+		const style::Checkbox &st = st::defaultCheckbox,
+		const style::Check &checkSt = st::defaultCheck);
 
 	bool checked() const {
 		return _checkbox->checked();
 	}
+	rpl::producer<bool> checkedChanges() const {
+		return _checkbox->checkedChanges();
+	}
+	rpl::producer<bool> checkedValue() const {
+		return _checkbox->checkedValue();
+	}
 
 	void setLabelText(
 		bool checked,
@@ -550,9 +614,8 @@ EditChatAdminsBoxController::LabeledCheckbox::LabeledCheckbox(
 	bool checked,
 	const style::Checkbox &st,
 	const style::Check &checkSt)
-: TWidget(parent)
+: RpWidget(parent)
 , _checkbox(this, text, checked, st, checkSt) {
-	subscribe(_checkbox->checkedChanged, [this](bool value) { checkedChanged.notify(value, true); });
 }
 
 void EditChatAdminsBoxController::LabeledCheckbox::setLabelText(
@@ -621,7 +684,8 @@ void EditChatAdminsBoxController::prepare() {
 		}));
 	}
 
-	subscribe(_allAdmins->checkedChanged, [this](bool checked) {
+	_allAdmins->checkedChanges(
+	) | rpl::start_with_next([=](bool checked) {
 		delegate()->peerListSetSearchMode(checked ? PeerListSearchMode::Disabled : PeerListSearchMode::Enabled);
 		for (auto i = 0, count = delegate()->peerListFullRowsCount(); i != count; ++i) {
 			auto row = delegate()->peerListRowAt(i);
@@ -632,7 +696,7 @@ void EditChatAdminsBoxController::prepare() {
 				row->setDisabledState(PeerListRow::State::Active);
 			}
 		}
-	});
+	}, _allAdmins->lifetime());
 }
 
 void EditChatAdminsBoxController::createAllAdminsCheckbox() {
@@ -656,7 +720,7 @@ void EditChatAdminsBoxController::rebuildRows() {
 	admins.reserve(allAdmins ? _chat->participants.size() : _chat->admins.size());
 	others.reserve(_chat->participants.size());
 
-	for (auto [user, version] : _chat->participants) {
+	for (const auto [user, version] : _chat->participants) {
 		if (user->id == peerFromUser(_chat->creator)) continue;
 		if (_chat->admins.contains(user)) {
 			admins.push_back(user);
@@ -751,36 +815,9 @@ void AddBotToGroupBoxController::rowClicked(not_null<PeerListRow*> row) {
 }
 
 void AddBotToGroupBoxController::shareBotGame(not_null<PeerData*> chat) {
-	auto send = [weak = base::make_weak(this), bot = _bot, chat] {
-		if (!weak) {
-			return;
-		}
-		const auto history = App::historyLoaded(chat);
-		const auto randomId = rand_value<uint64>();
-		const auto requestId = MTP::send(
-			MTPmessages_SendMedia(
-				MTP_flags(0),
-				chat->input,
-				MTP_int(0),
-				MTP_inputMediaGame(
-					MTP_inputGameShortName(
-						bot->inputUser,
-						MTP_string(bot->botInfo->shareGameShortName))),
-				MTP_string(""),
-				MTP_long(randomId),
-				MTPnullMarkup,
-				MTPnullEntities),
-			App::main()->rpcDone(&MainWidget::sentUpdatesReceived),
-			App::main()->rpcFail(&MainWidget::sendMessageFail),
-			0,
-			0,
-			history ? history->sendRequestId : 0);
-		if (history) {
-			history->sendRequestId = requestId;
-		}
-		Ui::hideLayer();
-		Ui::showPeerHistory(chat, ShowAtUnreadMsgId);
-	};
+	auto send = crl::guard(this, [bot = _bot, chat] {
+		ShareBotGame(bot, chat);
+	});
 	auto confirmText = [chat] {
 		if (chat->isUser()) {
 			return lng_bot_sure_share_game(lt_user, App::peerName(chat));
@@ -801,35 +838,9 @@ void AddBotToGroupBoxController::addBotToGroup(not_null<PeerData*> chat) {
 			return;
 		}
 	}
-	auto send = [weak = base::make_weak(this), bot = _bot, chat] {
-		if (!weak) {
-			return;
-		}
-		if (auto &info = bot->botInfo) {
-			if (!info->startGroupToken.isEmpty()) {
-				MTP::send(
-					MTPmessages_StartBot(
-						bot->inputUser,
-						chat->input,
-						MTP_long(rand_value<uint64>()),
-						MTP_string(info->startGroupToken)),
-					App::main()->rpcDone(&MainWidget::sentUpdatesReceived),
-					App::main()->rpcFail(
-						&MainWidget::addParticipantFail,
-						{ bot, chat }));
-			} else {
-				App::main()->addParticipants(
-					chat,
-					{ 1, bot });
-			}
-		} else {
-			App::main()->addParticipants(
-				chat,
-				{ 1, bot });
-		}
-		Ui::hideLayer();
-		Ui::showPeerHistory(chat, ShowAtUnreadMsgId);
-	};
+	auto send = crl::guard(this, [bot = _bot, chat] {
+		AddBotToGroup(bot, chat);
+	});
 	auto confirmText = lng_bot_sure_invite(lt_group, chat->name);
 	Ui::show(
 		Box<ConfirmBox>(confirmText, send),
@@ -908,7 +919,12 @@ void ChooseRecipientBoxController::prepareViewHook() {
 }
 
 void ChooseRecipientBoxController::rowClicked(not_null<PeerListRow*> row) {
-	_callback(row->peer());
+	auto weak = base::make_weak(this);
+	auto callback = std::move(_callback);
+	callback(row->peer());
+	if (weak) {
+		_callback = std::move(callback);
+	}
 }
 
 auto ChooseRecipientBoxController::createRow(
