@@ -10,9 +10,12 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "chat_helpers/emoji_suggestions_helper.h"
 #include "ui/effects/ripple_animation.h"
 #include "ui/widgets/shadow.h"
-#include "platform/platform_specific.h"
-#include "styles/style_chat_helpers.h"
 #include "ui/widgets/inner_dropdown.h"
+#include "ui/widgets/input_fields.h"
+#include "ui/emoji_config.h"
+#include "platform/platform_specific.h"
+#include "core/event_filter.h"
+#include "styles/style_chat_helpers.h"
 
 namespace Ui {
 namespace Emoji {
@@ -72,6 +75,14 @@ SuggestionsWidget::SuggestionsWidget(QWidget *parent, const style::Menu &st) : T
 	setMouseTracking(true);
 }
 
+rpl::producer<bool> SuggestionsWidget::toggleAnimated() const {
+	return _toggleAnimated.events();
+}
+
+rpl::producer<QString> SuggestionsWidget::triggered() const {
+	return _triggered.events();
+}
+
 void SuggestionsWidget::showWithQuery(const QString &query) {
 	if (_query == query) {
 		return;
@@ -79,7 +90,7 @@ void SuggestionsWidget::showWithQuery(const QString &query) {
 	_query = query;
 	auto rows = getRowsByQuery();
 	if (rows.empty()) {
-		toggleAnimated.notify(false, true);
+		_toggleAnimated.fire(false);
 	}
 	clearSelection();
 	_rows = std::move(rows);
@@ -89,7 +100,7 @@ void SuggestionsWidget::showWithQuery(const QString &query) {
 		setSelected(0);
 	}
 	if (!_rows.empty()) {
-		toggleAnimated.notify(true, true);
+		_toggleAnimated.fire(true);
 	}
 }
 
@@ -176,24 +187,29 @@ void SuggestionsWidget::paintEvent(QPaintEvent *e) {
 	if (clip.intersects(topskip)) p.fillRect(clip.intersected(topskip), _st->itemBg);
 	if (clip.intersects(bottomskip)) p.fillRect(clip.intersected(bottomskip), _st->itemBg);
 
-	auto top = _st->skip;
+	const auto top = _st->skip;
 	p.setFont(_st->itemFont);
-	auto from = floorclamp(clip.top() - top, _rowHeight, 0, _rows.size());
-	auto to = ceilclamp(clip.top() + clip.height() - top, _rowHeight, 0, _rows.size());
+	const auto from = floorclamp(clip.top() - top, _rowHeight, 0, _rows.size());
+	const auto to = ceilclamp(clip.top() + clip.height() - top, _rowHeight, 0, _rows.size());
 	p.translate(0, top + from * _rowHeight);
 	for (auto i = from; i != to; ++i) {
 		auto &row = _rows[i];
-		auto selected = (i == _selected || i == _pressed);
+		const auto selected = (i == _selected || i == _pressed);
 		p.fillRect(0, 0, width(), _rowHeight, selected ? _st->itemBgOver : _st->itemBg);
-		if (auto ripple = row.ripple()) {
+		if (const auto ripple = row.ripple()) {
 			ripple->paint(p, 0, 0, width(), ms);
 			if (ripple->empty()) {
 				row.resetRipple();
 			}
 		}
-		auto emoji = row.emoji();
-		auto esize = Ui::Emoji::Size(Ui::Emoji::Index() + 1);
-		p.drawPixmapLeft((_st->itemPadding.left() - (esize / cIntRetinaFactor())) / 2, (_rowHeight - (esize / cIntRetinaFactor())) / 2, width(), App::emojiLarge(), QRect(emoji->x() * esize, emoji->y() * esize, esize, esize));
+		const auto emoji = row.emoji();
+		const auto esize = Ui::Emoji::GetSizeLarge();
+		Ui::Emoji::Draw(
+			p,
+			emoji,
+			esize,
+			(_st->itemPadding.left() - (esize / cIntRetinaFactor())) / 2,
+			(_rowHeight - (esize / cIntRetinaFactor())) / 2);
 		p.setPen(selected ? _st->itemFgOver : _st->itemFg);
 		p.drawTextLeft(_st->itemPadding.left(), _st->itemPadding.top(), width(), row.label());
 		p.translate(0, _rowHeight);
@@ -330,7 +346,7 @@ void SuggestionsWidget::triggerSelectedRow() {
 }
 
 void SuggestionsWidget::triggerRow(const Row &row) {
-	triggered.notify(row.emoji()->text(), true);
+	_triggered.fire(row.emoji()->text());
 }
 
 void SuggestionsWidget::enterEventHook(QEvent *e) {
@@ -346,26 +362,68 @@ void SuggestionsWidget::leaveEventHook(QEvent *e) {
 	return TWidget::leaveEventHook(e);
 }
 
-SuggestionsController::SuggestionsController(QWidget *parent, not_null<QTextEdit*> field)
-: QObject(nullptr)
-, _field(field)
-, _container(parent, st::emojiSuggestionsDropdown)
-, _suggestions(_container->setOwnedWidget(object_ptr<Ui::Emoji::SuggestionsWidget>(parent, st::emojiSuggestionsMenu))) {
+SuggestionsController::SuggestionsController(
+	not_null<QWidget*> outer,
+	not_null<QTextEdit*> field)
+: _field(field) {
+	_container = base::make_unique_q<InnerDropdown>(
+		outer,
+		st::emojiSuggestionsDropdown);
 	_container->setAutoHiding(false);
+	_suggestions = _container->setOwnedWidget(
+		object_ptr<Ui::Emoji::SuggestionsWidget>(
+			_container,
+			st::emojiSuggestionsMenu));
 
 	setReplaceCallback(nullptr);
 
-	_field->installEventFilter(this);
-	connect(_field, &QTextEdit::textChanged, this, [this] { handleTextChange(); });
-	connect(_field, &QTextEdit::cursorPositionChanged, this, [this] { handleCursorPositionChange(); });
+	_fieldFilter.reset(Core::InstallEventFilter(
+		_field,
+		[=](not_null<QEvent*> event) { return fieldFilter(event); }));
+	_outerFilter.reset(Core::InstallEventFilter(
+		outer,
+		[=](not_null<QEvent*> event) { return outerFilter(event); }));
+	QObject::connect(
+		_field,
+		&QTextEdit::textChanged,
+		_container,
+		[=] { handleTextChange(); });
+	QObject::connect(
+		_field,
+		&QTextEdit::cursorPositionChanged,
+		_container,
+		[=] { handleCursorPositionChange(); });
+
+	_suggestions->toggleAnimated(
+	) | rpl::start_with_next([=](bool visible) {
+		suggestionsUpdated(visible);
+	}, _lifetime);
+	_suggestions->triggered(
+	) | rpl::start_with_next([=](QString replacement) {
+		replaceCurrent(replacement);
+	}, _lifetime);
 
-	subscribe(_suggestions->toggleAnimated, [this](bool visible) { suggestionsUpdated(visible); });
-	subscribe(_suggestions->triggered, [this](QString replacement) { replaceCurrent(replacement); });
 	updateForceHidden();
 
 	handleTextChange();
 }
 
+SuggestionsController *SuggestionsController::Init(
+		not_null<QWidget*> outer,
+		not_null<Ui::InputField*> field) {
+	const auto result = Ui::CreateChild<SuggestionsController>(
+		field.get(),
+		outer,
+		field->rawTextEdit());
+	result->setReplaceCallback([=](
+			int from,
+			int till,
+			const QString &replacement) {
+		field->commitInstantReplacement(from, till, replacement);
+	});
+	return result;
+}
+
 void SuggestionsController::setReplaceCallback(
 	Fn<void(
 		int from,
@@ -385,9 +443,9 @@ void SuggestionsController::setReplaceCallback(
 
 void SuggestionsController::handleTextChange() {
 	_ignoreCursorPositionChange = true;
-	InvokeQueued(this, [this] { _ignoreCursorPositionChange = false; });
+	InvokeQueued(_container, [=] { _ignoreCursorPositionChange = false; });
 
-	auto query = getEmojiQuery();
+	const auto query = getEmojiQuery();
 	if (query.isEmpty() || _textChangeAfterKeyPress) {
 		_suggestions->showWithQuery(query);
 	}
@@ -503,7 +561,7 @@ void SuggestionsController::replaceCurrent(const QString &replacement) {
 }
 
 void SuggestionsController::handleCursorPositionChange() {
-	InvokeQueued(this, [this] {
+	InvokeQueued(_container, [=] {
 		if (_ignoreCursorPositionChange) {
 			return;
 		}
@@ -517,7 +575,11 @@ void SuggestionsController::suggestionsUpdated(bool visible) {
 		_container->resizeToContent();
 		updateGeometry();
 		if (!_forceHidden) {
-			_container->showAnimated(Ui::PanelAnimation::Origin::BottomLeft);
+			if (_container->isHidden() || _container->isHiding()) {
+				raise();
+			}
+			_container->showAnimated(
+				Ui::PanelAnimation::Origin::BottomLeft);
 		}
 	} else if (!_forceHidden) {
 		_container->hideAnimated();
@@ -557,7 +619,7 @@ void SuggestionsController::updateGeometry() {
 }
 
 void SuggestionsController::updateForceHidden() {
-	_forceHidden = !_field->isVisible();
+	_forceHidden = !_field->isVisible() || !_field->hasFocus();
 	if (_forceHidden) {
 		_container->hideFast();
 	} else if (_shown) {
@@ -565,51 +627,68 @@ void SuggestionsController::updateForceHidden() {
 	}
 }
 
-bool SuggestionsController::eventFilter(QObject *object, QEvent *event) {
-	if (object == _field) {
-		auto type = event->type();
-		switch (type) {
-		case QEvent::Move:
-		case QEvent::Resize: {
+bool SuggestionsController::fieldFilter(not_null<QEvent*> event) {
+	auto type = event->type();
+	switch (type) {
+	case QEvent::Move:
+	case QEvent::Resize: {
+		if (_shown) {
+			updateGeometry();
+		}
+	} break;
+
+	case QEvent::Show:
+	case QEvent::ShowToParent:
+	case QEvent::Hide:
+	case QEvent::HideToParent:
+	case QEvent::FocusIn:
+	case QEvent::FocusOut: {
+		updateForceHidden();
+	} break;
+
+	case QEvent::KeyPress: {
+		const auto key = static_cast<QKeyEvent*>(event.get())->key();
+		switch (key) {
+		case Qt::Key_Enter:
+		case Qt::Key_Return:
+		case Qt::Key_Tab:
+		case Qt::Key_Up:
+		case Qt::Key_Down:
+			if (_shown && !_forceHidden) {
+				_suggestions->handleKeyEvent(key);
+				return true;
+			}
+			break;
+
+		case Qt::Key_Escape:
+			if (_shown && !_forceHidden) {
+				_suggestions->showWithQuery(QString());
+				return true;
+			}
+			break;
+		}
+		_textChangeAfterKeyPress = true;
+		InvokeQueued(_container, [=] { _textChangeAfterKeyPress = false; });
+	} break;
+	}
+	return false;
+}
+
+bool SuggestionsController::outerFilter(not_null<QEvent*> event) {
+	auto type = event->type();
+	switch (type) {
+	case QEvent::Move:
+	case QEvent::Resize: {
+		// updateGeometry uses not only container geometry, but also
+		// container children geometries that will be updated later.
+		InvokeQueued(_container, [=] {
 			if (_shown) {
 				updateGeometry();
 			}
-		} break;
-
-		case QEvent::Show:
-		case QEvent::ShowToParent:
-		case QEvent::Hide:
-		case QEvent::HideToParent: {
-			updateForceHidden();
-		} break;
-
-		case QEvent::KeyPress: {
-			auto key = static_cast<QKeyEvent*>(event)->key();
-			switch (key) {
-			case Qt::Key_Enter:
-			case Qt::Key_Return:
-			case Qt::Key_Tab:
-			case Qt::Key_Up:
-			case Qt::Key_Down:
-				if (_shown && !_forceHidden) {
-					_suggestions->handleKeyEvent(key);
-					return true;
-				}
-				break;
-
-			case Qt::Key_Escape:
-				if (_shown && !_forceHidden) {
-					_suggestions->showWithQuery(QString());
-					return true;
-				}
-				break;
-			}
-			_textChangeAfterKeyPress = true;
-			InvokeQueued(this, [this] { _textChangeAfterKeyPress = false; });
-		} break;
-		}
+		});
+	} break;
 	}
-	return QObject::eventFilter(object, event);
+	return false;
 }
 
 void SuggestionsController::raise() {
