@@ -8,10 +8,12 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #pragma once
 
 #include "ui/effects/panel_animation.h"
+#include "base/unique_qptr.h"
 
 namespace Ui {
 
 class InnerDropdown;
+class InputField;
 
 namespace Emoji {
 
@@ -22,8 +24,8 @@ public:
 	void showWithQuery(const QString &query);
 	void handleKeyEvent(int key);
 
-	base::Observable<bool> toggleAnimated;
-	base::Observable<QString> triggered;
+	rpl::producer<bool> toggleAnimated() const;
+	rpl::producer<QString> triggered() const;
 
 protected:
 	void paintEvent(QPaintEvent *e) override;
@@ -61,11 +63,16 @@ private:
 	int _selected = -1;
 	int _pressed = -1;
 
+	rpl::event_stream<bool> _toggleAnimated;
+	rpl::event_stream<QString> _triggered;
+
 };
 
-class SuggestionsController : public QObject, private base::Subscriber {
+class SuggestionsController {
 public:
-	SuggestionsController(QWidget *parent, not_null<QTextEdit*> field);
+	SuggestionsController(
+		not_null<QWidget*> outer,
+		not_null<QTextEdit*> field);
 
 	void raise();
 	void setReplaceCallback(Fn<void(
@@ -73,8 +80,9 @@ public:
 		int till,
 		const QString &replacement)> callback);
 
-protected:
-	bool eventFilter(QObject *object, QEvent *event) override;
+	static SuggestionsController *Init(
+		not_null<QWidget*> outer,
+		not_null<Ui::InputField*> field);
 
 private:
 	void handleCursorPositionChange();
@@ -84,6 +92,8 @@ private:
 	void updateGeometry();
 	void updateForceHidden();
 	void replaceCurrent(const QString &replacement);
+	bool fieldFilter(not_null<QEvent*> event);
+	bool outerFilter(not_null<QEvent*> event);
 
 	bool _shown = false;
 	bool _forceHidden = false;
@@ -95,8 +105,12 @@ private:
 		int from,
 		int till,
 		const QString &replacement)> _replaceCallback;
-	object_ptr<InnerDropdown> _container;
+	base::unique_qptr<InnerDropdown> _container;
 	QPointer<SuggestionsWidget> _suggestions;
+	base::unique_qptr<QObject> _fieldFilter;
+	base::unique_qptr<QObject> _outerFilter;
+
+	rpl::lifetime _lifetime;
 
 };
 
