@@ -26,6 +26,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "mainwindow.h"
 #include "window/window_controller.h"
 #include "storage/localstorage.h"
+#include "ui/image/image.h"
 #include "ui/empty_userpic.h"
 #include "ui/text_options.h"
 
@@ -171,13 +172,13 @@ void PeerData::setUserpic(
 }
 
 void PeerData::setUserpicPhoto(const MTPPhoto &data) {
-	auto photoId = [&]() -> PhotoId {
-		if (const auto photo = Auth().data().photo(data)) {
-			photo->peer = this;
-			return photo->id;
-		}
-		return 0;
-	}();
+	const auto photoId = data.match([&](const MTPDphoto &data) {
+		const auto photo = Auth().data().photo(data);
+		photo->peer = this;
+		return photo->id;
+	}, [](const MTPDphotoEmpty &data) {
+		return PhotoId(0);
+	});
 	if (_userpicPhotoId != photoId) {
 		_userpicPhotoId = photoId;
 		Notify::peerUpdatedDelayed(this, UpdateFlag::PhotoChanged);
@@ -186,7 +187,7 @@ void PeerData::setUserpicPhoto(const MTPPhoto &data) {
 
 ImagePtr PeerData::currentUserpic() const {
 	if (_userpic) {
-		_userpic->load(userpicPhotoOrigin());
+		_userpic->load(userpicOrigin());
 		if (_userpic->loaded()) {
 			if (!useEmptyUserpic()) {
 				_userpicEmpty = nullptr;
@@ -194,12 +195,15 @@ ImagePtr PeerData::currentUserpic() const {
 			return _userpic;
 		}
 	}
+	if (!_userpicEmpty) {
+		refreshEmptyUserpic();
+	}
 	return ImagePtr();
 }
 
 void PeerData::paintUserpic(Painter &p, int x, int y, int size) const {
 	if (auto userpic = currentUserpic()) {
-		p.drawPixmap(x, y, userpic->pixCircled(userpicPhotoOrigin(), size, size));
+		p.drawPixmap(x, y, userpic->pixCircled(userpicOrigin(), size, size));
 	} else {
 		_userpicEmpty->paint(p, x, y, x + size + x, size);
 	}
@@ -207,7 +211,7 @@ void PeerData::paintUserpic(Painter &p, int x, int y, int size) const {
 
 void PeerData::paintUserpicRounded(Painter &p, int x, int y, int size) const {
 	if (auto userpic = currentUserpic()) {
-		p.drawPixmap(x, y, userpic->pixRounded(userpicPhotoOrigin(), size, size, ImageRoundRadius::Small));
+		p.drawPixmap(x, y, userpic->pixRounded(userpicOrigin(), size, size, ImageRoundRadius::Small));
 	} else {
 		_userpicEmpty->paintRounded(p, x, y, x + size + x, size);
 	}
@@ -215,14 +219,31 @@ void PeerData::paintUserpicRounded(Painter &p, int x, int y, int size) const {
 
 void PeerData::paintUserpicSquare(Painter &p, int x, int y, int size) const {
 	if (auto userpic = currentUserpic()) {
-		p.drawPixmap(x, y, userpic->pix(userpicPhotoOrigin(), size, size));
+		p.drawPixmap(x, y, userpic->pix(userpicOrigin(), size, size));
 	} else {
 		_userpicEmpty->paintSquare(p, x, y, x + size + x, size);
 	}
 }
 
+void PeerData::loadUserpic(bool loadFirst, bool prior) {
+	_userpic->load(userpicOrigin(), loadFirst, prior);
+}
+
+bool PeerData::userpicLoaded() const {
+	return _userpic->loaded();
+}
+
+bool PeerData::useEmptyUserpic() const {
+	return _userpicLocation.isNull()
+		|| !_userpic
+		|| !_userpic->loaded();
+}
+
 StorageKey PeerData::userpicUniqueKey() const {
 	if (useEmptyUserpic()) {
+		if (!_userpicEmpty) {
+			refreshEmptyUserpic();
+		}
 		return _userpicEmpty->uniqueKey();
 	}
 	return storageKey(_userpicLocation);
@@ -238,7 +259,7 @@ void PeerData::saveUserpicRounded(const QString &path, int size) const {
 
 QPixmap PeerData::genUserpic(int size) const {
 	if (auto userpic = currentUserpic()) {
-		return userpic->pixCircled(userpicPhotoOrigin(), size, size);
+		return userpic->pixCircled(userpicOrigin(), size, size);
 	}
 	auto result = QImage(QSize(size, size) * cIntRetinaFactor(), QImage::Format_ARGB32_Premultiplied);
 	result.setDevicePixelRatio(cRetinaFactor());
@@ -252,7 +273,7 @@ QPixmap PeerData::genUserpic(int size) const {
 
 QPixmap PeerData::genUserpicRounded(int size) const {
 	if (auto userpic = currentUserpic()) {
-		return userpic->pixRounded(userpicPhotoOrigin(), size, size, ImageRoundRadius::Small);
+		return userpic->pixRounded(userpicOrigin(), size, size, ImageRoundRadius::Small);
 	}
 	auto result = QImage(QSize(size, size) * cIntRetinaFactor(), QImage::Format_ARGB32_Premultiplied);
 	result.setDevicePixelRatio(cRetinaFactor());
@@ -269,7 +290,7 @@ void PeerData::updateUserpic(
 		const MTPFileLocation &location) {
 	const auto size = kUserpicSize;
 	const auto loc = StorageImageLocation::FromMTP(size, size, location);
-	const auto photo = loc.isNull() ? ImagePtr() : ImagePtr(loc);
+	const auto photo = loc.isNull() ? ImagePtr() : Images::Create(loc);
 	setUserpicChecked(photoId, loc, photo);
 }
 
@@ -281,10 +302,9 @@ void PeerData::clearUserpic() {
 			auto image = Messenger::Instance().logoNoMargin().scaledToWidth(
 				kUserpicSize,
 				Qt::SmoothTransformation);
-			auto pixmap = App::pixmapFromImageInPlace(std::move(image));
 			return _userpic
 				? _userpic
-				: ImagePtr(std::move(pixmap), "PNG");
+				: Images::Create(std::move(image), "PNG");
 		}
 		return ImagePtr();
 	}();
@@ -296,7 +316,7 @@ void PeerData::setUserpicChecked(
 		const StorageImageLocation &location,
 		ImagePtr userpic) {
 	if (_userpicPhotoId != photoId
-		|| _userpic.v() != userpic.v()
+		|| _userpic.get() != userpic.get()
 		|| _userpicLocation != location) {
 		setUserpic(photoId, location, userpic);
 		Notify::peerUpdatedDelayed(this, UpdateFlag::PhotoChanged);
@@ -310,6 +330,39 @@ void PeerData::setUserpicChecked(
 	}
 }
 
+bool PeerData::canPinMessages() const {
+	if (const auto user = asUser()) {
+		return user->fullFlags() & MTPDuserFull::Flag::f_can_pin_message;
+	} else if (const auto chat = asChat()) {
+		return chat->adminsEnabled() ? chat->amAdmin() : chat->amIn();
+	} else if (const auto channel = asChannel()) {
+		using AdminRight = ChannelData::AdminRight;
+		if (channel->isMegagroup()) {
+			return (channel->adminRights() & AdminRight::f_pin_messages)
+				|| channel->amCreator();
+		}
+		return (channel->adminRights() & AdminRight::f_edit_messages)
+			|| channel->amCreator();
+	}
+	Unexpected("Peer type in PeerData::canPinMessages.");
+}
+
+void PeerData::setPinnedMessageId(MsgId messageId) {
+	const auto min = [&] {
+		if (const auto channel = asChannel()) {
+			return channel->availableMinId();
+		}
+		return MsgId(0);
+	}();
+	messageId = (messageId > min) ? messageId : MsgId(0);
+	if (_pinnedMessageId != messageId) {
+		_pinnedMessageId = messageId;
+		Notify::peerUpdatedDelayed(
+			this,
+			Notify::PeerUpdate::Flag::PinnedMessageChanged);
+	}
+}
+
 void PeerData::fillNames() {
 	_nameWords.clear();
 	_nameFirstLetters.clear();
@@ -332,7 +385,12 @@ void PeerData::fillNames() {
 		}
 		appendToIndex(user->username);
 		if (isSelf()) {
-			appendToIndex(lang(lng_saved_messages));
+			const auto english = qsl("Saved messages");
+			const auto localized = lang(lng_saved_messages);
+			appendToIndex(english);
+			if (localized != english) {
+				appendToIndex(localized);
+			}
 		}
 	} else if (const auto channel = asChannel()) {
 		appendToIndex(channel->username);
@@ -852,25 +910,12 @@ void ChannelData::setAvailableMinId(MsgId availableMinId) {
 		if (auto history = App::historyLoaded(this)) {
 			history->clearUpTill(availableMinId);
 		}
-		if (_pinnedMessageId <= _availableMinId) {
-			_pinnedMessageId = MsgId(0);
-			Notify::peerUpdatedDelayed(
-				this,
-				Notify::PeerUpdate::Flag::ChannelPinnedChanged);
+		if (pinnedMessageId() <= _availableMinId) {
+			clearPinnedMessage();
 		}
 	}
 }
 
-void ChannelData::setPinnedMessageId(MsgId messageId) {
-	messageId = (messageId > _availableMinId) ? messageId : MsgId(0);
-	if (_pinnedMessageId != messageId) {
-		_pinnedMessageId = messageId;
-		Notify::peerUpdatedDelayed(
-			this,
-			Notify::PeerUpdate::Flag::ChannelPinnedChanged);
-	}
-}
-
 void ChannelData::setFeed(not_null<Data::Feed*> feed) {
 	setFeedPointer(feed);
 }
@@ -928,15 +973,6 @@ bool ChannelData::canAddAdmins() const {
 		|| amCreator();
 }
 
-bool ChannelData::canPinMessages() const {
-	if (isMegagroup()) {
-		return (adminRights() & AdminRight::f_pin_messages)
-			|| amCreator();
-	}
-	return (adminRights() & AdminRight::f_edit_messages)
-		|| amCreator();
-}
-
 bool ChannelData::canPublish() const {
 	return (adminRights() & AdminRight::f_post_messages)
 		|| amCreator();
