@@ -11,10 +11,13 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "chat_helpers/stickers.h"
 #include "dialogs/dialogs_key.h"
 #include "data/data_groups.h"
+#include "history/history_location_manager.h"
 #include "base/timer.h"
 
 class HistoryItem;
 class BoxContent;
+struct WebPageCollage;
+enum class WebPageType;
 
 namespace HistoryView {
 struct Group;
@@ -261,14 +264,17 @@ public:
 	void photoConvert(
 		not_null<PhotoData*> original,
 		const MTPPhoto &data);
-	PhotoData *photoFromWeb(const MTPWebDocument &data, ImagePtr thumb);
+	PhotoData *photoFromWeb(
+		const MTPWebDocument &data,
+		ImagePtr thumb = ImagePtr(),
+		bool willBecomeNormal = false);
 
 	not_null<DocumentData*> document(DocumentId id);
 	not_null<DocumentData*> document(const MTPDocument &data);
 	not_null<DocumentData*> document(const MTPDdocument &data);
 	not_null<DocumentData*> document(
 		const MTPdocument &data,
-		const QPixmap &thumb);
+		QImage &&thumb);
 	not_null<DocumentData*> document(
 		DocumentId id,
 		const uint64 &access,
@@ -297,7 +303,7 @@ public:
 		const TextWithEntities &content);
 	not_null<WebPageData*> webpage(
 		WebPageId id,
-		const QString &type,
+		WebPageType type,
 		const QString &url,
 		const QString &displayUrl,
 		const QString &siteName,
@@ -305,6 +311,7 @@ public:
 		const TextWithEntities &description,
 		PhotoData *photo,
 		DocumentData *document,
+		WebPageCollage &&collage,
 		int duration,
 		const QString &author,
 		TimeId pendingTill);
@@ -323,6 +330,8 @@ public:
 		not_null<GameData*> original,
 		const MTPGame &data);
 
+	not_null<LocationData*> location(const LocationCoords &coords);
+
 	void registerPhotoItem(
 		not_null<const PhotoData*> photo,
 		not_null<HistoryItem*> item);
@@ -407,14 +416,15 @@ public:
 	bool notifySettingsUnknown(not_null<const PeerData*> peer) const;
 	rpl::producer<> defaultUserNotifyUpdates() const;
 	rpl::producer<> defaultChatNotifyUpdates() const;
+	rpl::producer<> defaultBroadcastNotifyUpdates() const;
 	rpl::producer<> defaultNotifyUpdates(
 		not_null<const PeerData*> peer) const;
 
 	void serviceNotification(
 		const TextWithEntities &message,
-		const MTPMessageMedia &media);
-
-	void forgetMedia();
+		const MTPMessageMedia &media = MTP_messageMediaEmpty());
+	void checkNewAuthorization();
+	rpl::producer<> newAuthorizationChecks() const;
 
 	void setMimeForwardIds(MessageIdsList &&list);
 	MessageIdsList takeMimeForwardIds();
@@ -478,7 +488,7 @@ private:
 		const MTPDwebPage &data);
 	void webpageApplyFields(
 		not_null<WebPageData*> page,
-		const QString &type,
+		WebPageType type,
 		const QString &url,
 		const QString &displayUrl,
 		const QString &siteName,
@@ -486,6 +496,7 @@ private:
 		const TextWithEntities &description,
 		PhotoData *photo,
 		DocumentData *document,
+		WebPageCollage &&collage,
 		int duration,
 		const QString &author,
 		TimeId pendingTill);
@@ -593,6 +604,9 @@ private:
 	std::unordered_map<
 		WebPageId,
 		std::unique_ptr<WebPageData>> _webpages;
+	std::unordered_map<
+		LocationCoords,
+		std::unique_ptr<LocationData>> _locations;
 	std::map<
 		not_null<const WebPageData*>,
 		base::flat_set<not_null<HistoryItem*>>> _webpageItems;
@@ -630,8 +644,10 @@ private:
 
 	NotifySettings _defaultUserNotifySettings;
 	NotifySettings _defaultChatNotifySettings;
+	NotifySettings _defaultBroadcastNotifySettings;
 	rpl::event_stream<> _defaultUserNotifyUpdates;
 	rpl::event_stream<> _defaultChatNotifyUpdates;
+	rpl::event_stream<> _defaultBroadcastNotifyUpdates;
 	std::unordered_set<not_null<const PeerData*>> _mutedPeers;
 	base::Timer _unmuteByFinishedTimer;
 
@@ -642,6 +658,8 @@ private:
 		int>;
 	std::unique_ptr<CredentialsWithGeneration> _passportCredentials;
 
+	rpl::event_stream<> _newAuthorizationChecks;
+
 	rpl::lifetime _lifetime;
 
 };
