@@ -830,14 +830,20 @@ Result TextWriter::writeDialogSlice(const Data::MessagesSlice &data) {
 	Expects(_chat != nullptr);
 	Expects(!data.list.empty());
 
-	_messagesCount += data.list.size();
 	auto list = std::vector<QByteArray>();
 	list.reserve(data.list.size());
 	for (const auto &message : data.list) {
+		if (Data::SkipMessageByDate(message, _settings)) {
+			continue;
+		}
 		list.push_back(SerializeMessage(
 			message,
 			data.peers,
 			_environment.internalLinksDomain));
+		++_messagesCount;
+	}
+	if (list.empty()) {
+		return Result::Success();
 	}
 	const auto full = _chat->empty()
 		? JoinList(kLineBreak, list)
