@@ -18,6 +18,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "styles/style_dialogs.h"
 #include "data/data_drafts.h"
 #include "data/data_session.h"
+#include "data/data_media_types.h"
 #include "lang/lang_keys.h"
 #include "apiwrap.h"
 #include "mainwidget.h"
@@ -32,6 +33,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "storage/storage_feed_messages.h"
 #include "data/data_channel_admins.h"
 #include "data/data_feed.h"
+#include "ui/image/image.h"
 #include "ui/text_options.h"
 #include "core/crash_reports.h"
 
@@ -106,6 +108,7 @@ void Histories::clear() {
 	_map.clear();
 
 	_unreadFull = _unreadMuted = 0;
+	_unreadEntriesFull = _unreadEntriesMuted = 0;
 	Notify::unreadCounterUpdated();
 	App::historyClearItems();
 	typing.clear();
@@ -162,34 +165,135 @@ HistoryItem *Histories::addNewMessage(
 }
 
 int Histories::unreadBadge() const {
-	return _unreadFull - (Global::IncludeMuted() ? 0 : _unreadMuted);
+	return computeUnreadBadge(
+		_unreadFull,
+		_unreadMuted,
+		_unreadEntriesFull,
+		_unreadEntriesMuted);
 }
 
-int Histories::unreadMutedCount() const {
-	return _unreadMuted;
+bool Histories::unreadBadgeMuted() const {
+	return computeUnreadBadgeMuted(
+		_unreadFull,
+		_unreadMuted,
+		_unreadEntriesFull,
+		_unreadEntriesMuted);
+}
+
+int Histories::unreadBadgeIgnoreOne(History *history) const {
+	const auto removeCount = (history
+		&& history->inChatList(Dialogs::Mode::All))
+		? history->unreadCount()
+		: 0;
+	if (!removeCount) {
+		return unreadBadge();
+	}
+	const auto removeMuted = history->mute();
+	return computeUnreadBadge(
+		_unreadFull - removeCount,
+		_unreadMuted - (removeMuted ? removeCount : 0),
+		_unreadEntriesFull - 1,
+		_unreadEntriesMuted - (removeMuted ? 1 : 0));
+}
+
+bool Histories::unreadBadgeMutedIgnoreOne(History *history) const {
+	const auto removeCount = (history
+		&& history->inChatList(Dialogs::Mode::All))
+		? history->unreadCount()
+		: 0;
+	if (!removeCount) {
+		return unreadBadgeMuted();
+	}
+	const auto removeMuted = history->mute();
+	return computeUnreadBadgeMuted(
+		_unreadFull - removeCount,
+		_unreadMuted - (removeMuted ? removeCount : 0),
+		_unreadEntriesFull - 1,
+		_unreadEntriesMuted - (removeMuted ? 1 : 0));
+}
+
+int Histories::unreadOnlyMutedBadge() const {
+	return Auth().settings().countUnreadMessages()
+		? _unreadMuted
+		: _unreadEntriesMuted;
+}
+
+int Histories::computeUnreadBadge(
+		int full,
+		int muted,
+		int entriesFull,
+		int entriesMuted) const {
+	const auto withMuted = Auth().settings().includeMutedCounter();
+	return Auth().settings().countUnreadMessages()
+		? (full - (withMuted ? 0 : muted))
+		: (entriesFull - (withMuted ? 0 : entriesMuted));
+}
+
+bool Histories::computeUnreadBadgeMuted(
+		int full,
+		int muted,
+		int entriesFull,
+		int entriesMuted) const {
+	if (!Auth().settings().includeMutedCounter()) {
+		return false;
+	}
+	return Auth().settings().countUnreadMessages()
+		? (muted >= full)
+		: (entriesMuted >= entriesFull);
 }
 
 void Histories::unreadIncrement(int count, bool muted) {
+	if (!count) {
+		return;
+	}
 	_unreadFull += count;
 	if (muted) {
 		_unreadMuted += count;
 	}
-	if (!muted || Global::IncludeMuted()) {
-		Notify::unreadCounterUpdated();
+	if (Auth().settings().countUnreadMessages()) {
+		if (!muted || Auth().settings().includeMutedCounter()) {
+			Notify::unreadCounterUpdated();
+		}
 	}
 }
 
 void Histories::unreadMuteChanged(int count, bool muted) {
+	const auto wasAll = (_unreadMuted == _unreadFull);
 	if (muted) {
 		_unreadMuted += count;
 	} else {
 		_unreadMuted -= count;
 	}
-	Notify::unreadCounterUpdated();
+	if (Auth().settings().countUnreadMessages()) {
+		const auto nowAll = (_unreadMuted == _unreadFull);
+		const auto changed = !Auth().settings().includeMutedCounter()
+			|| (wasAll != nowAll);
+		if (changed) {
+			Notify::unreadCounterUpdated();
+		}
+	}
 }
 
-bool Histories::unreadOnlyMuted() const {
-	return Global::IncludeMuted() ? (_unreadMuted >= _unreadFull) : false;
+void Histories::unreadEntriesChanged(
+		int withUnreadDelta,
+		int mutedWithUnreadDelta) {
+	if (!withUnreadDelta && !mutedWithUnreadDelta) {
+		return;
+	}
+	const auto wasAll = (_unreadEntriesMuted == _unreadEntriesFull);
+	_unreadEntriesFull += withUnreadDelta;
+	_unreadEntriesMuted += mutedWithUnreadDelta;
+	if (!Auth().settings().countUnreadMessages()) {
+		const auto nowAll = (_unreadEntriesMuted == _unreadEntriesFull);
+		const auto withMuted = Auth().settings().includeMutedCounter();
+		const auto withMutedChanged = withMuted
+			&& (withUnreadDelta != 0 || wasAll != nowAll);
+		const auto withoutMutedChanged = !withMuted
+			&& (withUnreadDelta != mutedWithUnreadDelta);
+		if (withMutedChanged || withoutMutedChanged) {
+			Notify::unreadCounterUpdated();
+		}
+	}
 }
 
 void Histories::selfDestructIn(not_null<HistoryItem*> item, TimeMs delay) {
@@ -321,10 +425,8 @@ void History::itemVanished(not_null<HistoryItem*> item) {
 		&& unreadCount() > 0) {
 		changeUnreadCount(-1);
 	}
-	if (const auto channel = peer->asChannel()) {
-		if (channel->pinnedMessageId() == item->id) {
-			channel->clearPinnedMessage();
-		}
+	if (peer->pinnedMessageId() == item->id) {
+		peer->clearPinnedMessage();
 	}
 }
 
@@ -348,12 +450,14 @@ void History::takeLocalDraft(History *from) {
 
 void History::createLocalDraftFromCloud() {
 	auto draft = cloudDraft();
-	if (Data::draftIsNull(draft) || !draft->date) {
+	if (Data::draftIsNull(draft) || !draft->date || Auth().supportMode()) {
 		return;
 	}
 
 	auto existing = localDraft();
-	if (Data::draftIsNull(existing) || !existing->date || draft->date >= existing->date) {
+	if (Data::draftIsNull(existing)
+		|| !existing->date
+		|| draft->date >= existing->date) {
 		if (!existing) {
 			setLocalDraft(std::make_unique<Data::Draft>(
 				draft->textWithTags,
@@ -376,7 +480,7 @@ void History::setCloudDraft(std::unique_ptr<Data::Draft> &&draft) {
 	cloudDraftTextCache.clear();
 }
 
-Data::Draft *History::createCloudDraft(Data::Draft *fromDraft) {
+Data::Draft *History::createCloudDraft(const Data::Draft *fromDraft) {
 	if (Data::draftIsNull(fromDraft)) {
 		setCloudDraft(std::make_unique<Data::Draft>(
 			TextWithTags(),
@@ -843,11 +947,17 @@ not_null<HistoryItem*> History::addNewGame(
 }
 
 void History::setUnreadMentionsCount(int count) {
+	const auto had = _unreadMentionsCount && (*_unreadMentionsCount > 0);
 	if (_unreadMentions.size() > count) {
 		LOG(("API Warning: real mentions count is greater than received mentions count"));
 		count = _unreadMentions.size();
 	}
 	_unreadMentionsCount = count;
+	const auto has = (count > 0);
+	if (has != had && Global::DialogsModeEnabled()) {
+		Notify::historyMuteUpdated(this);
+		updateChatListEntry();
+	}
 }
 
 bool History::addToUnreadMentions(
@@ -861,8 +971,8 @@ bool History::addToUnreadMentions(
 		: false;
 	if (allLoaded) {
 		if (type == UnreadMentionType::New) {
-			++*_unreadMentionsCount;
 			_unreadMentions.insert(msgId);
+			setUnreadMentionsCount(*_unreadMentionsCount + 1);
 			return true;
 		}
 	} else if (!_unreadMentions.empty() && type != UnreadMentionType::New) {
@@ -874,10 +984,8 @@ bool History::addToUnreadMentions(
 
 void History::eraseFromUnreadMentions(MsgId msgId) {
 	_unreadMentions.remove(msgId);
-	if (_unreadMentionsCount) {
-		if (*_unreadMentionsCount > 0) {
-			--*_unreadMentionsCount;
-		}
+	if (_unreadMentionsCount && *_unreadMentionsCount > 0) {
+		setUnreadMentionsCount(*_unreadMentionsCount - 1);
 	}
 	Notify::peerUpdatedDelayed(peer, Notify::PeerUpdate::Flag::UnreadMentionsChanged);
 }
@@ -1214,9 +1322,7 @@ void History::applyServiceChanges(
 
 	case mtpc_messageActionPinMessage: {
 		if (data.has_reply_to_msg_id() && item) {
-			if (auto channel = item->history()->peer->asChannel()) {
-				channel->setPinnedMessageId(data.vreply_to_msg_id.v);
-			}
+			item->history()->peer->setPinnedMessageId(data.vreply_to_msg_id.v);
 		}
 	} break;
 
@@ -1261,7 +1367,6 @@ void History::mainViewRemoved(
 }
 
 void History::newItemAdded(not_null<HistoryItem*> item) {
-	App::checkImageCacheSize();
 	item->indexAsNewItem();
 	if (const auto from = item->from() ? item->from()->asUser() : nullptr) {
 		if (from == item->author()) {
@@ -1656,6 +1761,7 @@ bool History::unreadCountKnown() const {
 
 void History::setUnreadCount(int newUnreadCount) {
 	if (!_unreadCount || *_unreadCount != newUnreadCount) {
+		const auto wasUnread = _unreadMark || unreadCount();
 		const auto unreadCountDelta = _unreadCount | [&](int count) {
 			return newUnreadCount - count;
 		};
@@ -1700,12 +1806,20 @@ void History::setUnreadCount(int newUnreadCount) {
 		}
 
 		if (inChatList(Dialogs::Mode::All)) {
-			const auto delta = unreadCountDelta
+			const auto delta = unreadMarkDelta + (unreadCountDelta
 				? *unreadCountDelta
-				: newUnreadCount;
-			App::histories().unreadIncrement(
-				delta + unreadMarkDelta,
-				mute());
+				: newUnreadCount);
+			App::histories().unreadIncrement(delta, mute());
+
+			const auto nowUnread = (*_unreadCount > 0) || _unreadMark;
+			const auto entriesDelta = (wasUnread && !nowUnread)
+				? -1
+				: (nowUnread && !wasUnread)
+				? 1
+				: 0;
+			App::histories().unreadEntriesChanged(
+				entriesDelta,
+				mute() ? entriesDelta : 0);
 		}
 		Notify::peerUpdatedDelayed(
 			peer,
@@ -1714,12 +1828,18 @@ void History::setUnreadCount(int newUnreadCount) {
 }
 
 void History::setUnreadMark(bool unread) {
+	if (clearUnreadOnClientSide()) {
+		unread = false;
+	}
 	if (_unreadMark != unread) {
 		_unreadMark = unread;
 		if (!_unreadCount || !*_unreadCount) {
 			if (inChatList(Dialogs::Mode::All)) {
 				const auto delta = _unreadMark ? 1 : -1;
 				App::histories().unreadIncrement(delta, mute());
+				App::histories().unreadEntriesChanged(
+					delta,
+					mute() ? delta : 0);
 
 				updateChatListEntry();
 			}
@@ -1774,6 +1894,13 @@ bool History::changeMute(bool newMute) {
 	if (inChatList(Dialogs::Mode::All)) {
 		if (const auto count = historiesUnreadCount()) {
 			App::histories().unreadMuteChanged(count, _mute);
+
+			const auto entriesWithUnreadDelta = 0;
+			const auto mutedEntriesWithUnreadDelta = _mute ? 1 : -1;
+			App::histories().unreadEntriesChanged(
+				entriesWithUnreadDelta,
+				mutedEntriesWithUnreadDelta);
+
 			Notify::unreadCounterUpdated();
 		}
 		Notify::historyMuteUpdated(this);
@@ -1830,7 +1957,7 @@ std::shared_ptr<AdminLog::LocalIdManager> History::adminLogIdManager() {
 TimeId History::adjustChatListTimeId() const {
 	const auto result = chatsListTimeId();
 	if (const auto draft = cloudDraft()) {
-		if (!Data::draftIsNull(draft)) {
+		if (!Data::draftIsNull(draft) && !Auth().supportMode()) {
 			return std::max(result, draft->date);
 		}
 	}
@@ -2239,6 +2366,10 @@ bool History::shouldBeInChatList() const {
 	return true;
 }
 
+bool History::toImportant() const {
+	return !mute() || hasUnreadMentions();
+}
+
 void History::unknownMessageDeleted(MsgId messageId) {
 	if (_inboxReadBefore && messageId >= *_inboxReadBefore) {
 		changeUnreadCount(-1);
@@ -2285,6 +2416,18 @@ void History::applyDialog(const MTPDdialog &data) {
 	}
 }
 
+bool History::clearUnreadOnClientSide() const {
+	if (!Auth().supportMode()) {
+		return false;
+	}
+	if (const auto user = peer->asUser()) {
+		if (user->flags() & MTPDuser::Flag::f_deleted) {
+			return true;
+		}
+	}
+	return false;
+}
+
 bool History::skipUnreadUpdateForClientSideUnread() const {
 	if (peer->id != peerFromUser(ServiceUserId)) {
 		return false;
@@ -2296,11 +2439,16 @@ bool History::skipUnreadUpdateForClientSideUnread() const {
 	return true;
 }
 
+bool History::skipUnreadUpdate() const {
+	return skipUnreadUpdateForClientSideUnread()
+		|| clearUnreadOnClientSide();
+}
+
 void History::applyDialogFields(
 		int unreadCount,
 		MsgId maxInboxRead,
 		MsgId maxOutboxRead) {
-	if (!skipUnreadUpdateForClientSideUnread()) {
+	if (!skipUnreadUpdate()) {
 		setUnreadCount(unreadCount);
 		setInboxReadTill(maxInboxRead);
 	}
@@ -2320,6 +2468,12 @@ void History::applyDialogTopMessage(MsgId topMessageId) {
 	} else {
 		setLastMessage(nullptr);
 	}
+	if (clearUnreadOnClientSide()) {
+		setUnreadCount(0);
+		if (const auto last = lastMessage()) {
+			setInboxReadTill(last->id);
+		}
+	}
 }
 
 void History::setInboxReadTill(MsgId upTo) {
@@ -2379,6 +2533,13 @@ HistoryItem *History::lastSentMessage() const {
 	for (const auto &block : base::reversed(blocks)) {
 		for (const auto &message : base::reversed(block->messages)) {
 			const auto item = message->data();
+			// Skip if message is video message or sticker.
+			if (const auto media = item->media()) {
+				// Skip only if media is not webpage.
+				if (!media->webpage() && !media->allowsEditCaption()) {
+					continue;
+				}
+			}
 			if (IsServerMsgId(item->id)
 				&& !item->serviceMsg()
 				&& (item->out() || peer->isSelf())) {
@@ -2726,6 +2887,11 @@ void History::changedInChatListHook(Dialogs::Mode list, bool added) {
 	if (list == Dialogs::Mode::All) {
 		if (const auto delta = historiesUnreadCount() * (added ? 1 : -1)) {
 			App::histories().unreadIncrement(delta, mute());
+
+			const auto entriesDelta = added ? 1 : -1;
+			App::histories().unreadEntriesChanged(
+				entriesDelta,
+				mute() ? entriesDelta : 0);
 		}
 	}
 }
@@ -2733,7 +2899,7 @@ void History::changedInChatListHook(Dialogs::Mode list, bool added) {
 void History::changedChatListPinHook() {
 	Notify::peerUpdatedDelayed(
 		peer,
-		Notify::PeerUpdate::Flag::PinnedChanged);
+		Notify::PeerUpdate::Flag::ChatPinnedChanged);
 }
 
 void History::removeBlock(not_null<HistoryBlock*> block) {
