@@ -16,6 +16,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "history/history_item.h"
 #include "data/data_media_types.h"
 #include "window/window_controller.h"
+#include "core/shortcuts.h"
 #include "messenger.h"
 #include "mainwindow.h"
 #include "auth_session.h"
@@ -56,22 +57,27 @@ Instance::Instance()
 	});
 
 	// While we have one Media::Player::Instance for all authsessions we have to do this.
-	auto handleAuthSessionChange = [this] {
+	const auto handleAuthSessionChange = [=] {
 		if (AuthSession::Exists()) {
-			subscribe(Auth().calls().currentCallChanged(), [this](Calls::Call *call) {
+			subscribe(Auth().calls().currentCallChanged(), [=](Calls::Call *call) {
 				if (call) {
-					pause(AudioMsgId::Type::Voice);
-					pause(AudioMsgId::Type::Song);
+					pauseOnCall(AudioMsgId::Type::Voice);
+					pauseOnCall(AudioMsgId::Type::Song);
+				} else {
+					resumeOnCall(AudioMsgId::Type::Voice);
+					resumeOnCall(AudioMsgId::Type::Song);
 				}
 			});
 		} else {
 			handleLogout();
 		}
 	};
-	subscribe(Messenger::Instance().authSessionChanged(), [=] {
-		handleAuthSessionChange();
-	});
+	subscribe(
+		Messenger::Instance().authSessionChanged(),
+		handleAuthSessionChange);
 	handleAuthSessionChange();
+
+	setupShortcuts();
 }
 
 AudioMsgId::Type Instance::getActiveType() const {
@@ -289,6 +295,9 @@ void Instance::play(AudioMsgId::Type type) {
 			play(data->current);
 		}
 	}
+	if (const auto data = getData(type)) {
+		data->resumeOnCallEnd = false;
+	}
 }
 
 void Instance::play(const AudioMsgId &audioId) {
@@ -313,21 +322,24 @@ void Instance::play(const AudioMsgId &audioId) {
 }
 
 void Instance::pause(AudioMsgId::Type type) {
-	auto state = mixer()->currentState(type);
+	const auto state = mixer()->currentState(type);
 	if (state.id) {
 		mixer()->pause(state.id);
 	}
 }
 
 void Instance::stop(AudioMsgId::Type type) {
-	auto state = mixer()->currentState(type);
+	const auto state = mixer()->currentState(type);
 	if (state.id) {
 		mixer()->stop(state.id);
 	}
+	if (const auto data = getData(type)) {
+		data->resumeOnCallEnd = false;
+	}
 }
 
 void Instance::playPause(AudioMsgId::Type type) {
-	auto state = mixer()->currentState(type);
+	const auto state = mixer()->currentState(type);
 	if (state.id) {
 		if (IsStopped(state.state)) {
 			play(state.id);
@@ -341,17 +353,43 @@ void Instance::playPause(AudioMsgId::Type type) {
 			play(data->current);
 		}
 	}
+	if (const auto data = getData(type)) {
+		data->resumeOnCallEnd = false;
+	}
+}
+
+void Instance::pauseOnCall(AudioMsgId::Type type) {
+	const auto state = mixer()->currentState(type);
+	if (!state.id
+		|| IsStopped(state.state)
+		|| IsPaused(state.state)
+		|| state.state == State::Pausing) {
+		return;
+	}
+	pause(type);
+	if (const auto data = getData(type)) {
+		data->resumeOnCallEnd = true;
+	}
+}
+
+void Instance::resumeOnCall(AudioMsgId::Type type) {
+	if (const auto data = getData(type)) {
+		if (data->resumeOnCallEnd) {
+			data->resumeOnCallEnd = false;
+			play(type);
+		}
+	}
 }
 
 bool Instance::next(AudioMsgId::Type type) {
-	if (auto data = getData(type)) {
+	if (const auto data = getData(type)) {
 		return moveInPlaylist(data, 1, false);
 	}
 	return false;
 }
 
 bool Instance::previous(AudioMsgId::Type type) {
-	if (auto data = getData(type)) {
+	if (const auto data = getData(type)) {
 		return moveInPlaylist(data, -1, false);
 	}
 	return false;
@@ -459,5 +497,36 @@ void Instance::handleLogout() {
 	_usePanelPlayer.notify(false, true);
 }
 
+void Instance::setupShortcuts() {
+	Shortcuts::Requests(
+	) | rpl::start_with_next([=](not_null<Shortcuts::Request*> request) {
+		using Command = Shortcuts::Command;
+		request->check(Command::MediaPlay) && request->handle([=] {
+			play();
+			return true;
+		});
+		request->check(Command::MediaPause) && request->handle([=] {
+			pause();
+			return true;
+		});
+		request->check(Command::MediaPlayPause) && request->handle([=] {
+			playPause();
+			return true;
+		});
+		request->check(Command::MediaStop) && request->handle([=] {
+			stop();
+			return true;
+		});
+		request->check(Command::MediaPrevious) && request->handle([=] {
+			previous();
+			return true;
+		});
+		request->check(Command::MediaNext) && request->handle([=] {
+			next();
+			return true;
+		});
+	}, _lifetime);
+}
+
 } // namespace Player
 } // namespace Media
