@@ -13,8 +13,10 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_document.h"
 #include "data/data_media_types.h"
 #include "data/data_session.h"
+#include "data/data_web_page.h"
 #include "history/history.h"
 #include "history/history_media.h"
+#include "ui/image/image.h"
 #include "auth_session.h"
 #include "styles/style_mediaview.h"
 
@@ -37,11 +39,17 @@ Data::FileOrigin ComputeFileOrigin(const Key &key, const Context &context) {
 			return peerIsUser(peerId)
 				? Data::FileOriginUserPhoto(peerToUser(peerId), photoId)
 				: Data::FileOrigin(Data::FileOriginPeerPhoto(peerId));
-		}, [&](auto&&) {
+		}, [](auto&&) {
 			return Data::FileOrigin();
 		});
 	}, [](FullMsgId itemId) {
 		return Data::FileOrigin(itemId);
+	}, [&](GroupThumbs::CollageKey) {
+		return context.match([](const GroupThumbs::CollageSlice &slice) {
+			return Data::FileOrigin(slice.context);
+		}, [](auto&&) {
+			return Data::FileOrigin();
+		});
 	});
 }
 
@@ -71,6 +79,10 @@ Context ComputeContext(const UserPhotosSlice &slice, int index) {
 	return peerFromUser(slice.key().userId);
 }
 
+Context ComputeContext(const GroupThumbs::CollageSlice &slice, int index) {
+	return slice.context;
+}
+
 Key ComputeKey(const SharedMediaWithLastSlice &slice, int index) {
 	Expects(index >= 0 && index < slice.size());
 
@@ -87,6 +99,10 @@ Key ComputeKey(const UserPhotosSlice &slice, int index) {
 	return slice[index];
 }
 
+Key ComputeKey(const GroupThumbs::CollageSlice &slice, int index) {
+	return GroupThumbs::CollageKey{ index };
+}
+
 int ComputeThumbsLimit(int availableWidth) {
 	const auto singleWidth = st::mediaviewGroupWidth
 		+ 2 * st::mediaviewGroupSkip;
@@ -350,6 +366,10 @@ ClickHandlerPtr GroupThumbs::Thumb::getState(QPoint point) const {
 		: nullptr;
 }
 
+int GroupThumbs::CollageSlice::size() const {
+	return data->items.size();
+}
+
 GroupThumbs::GroupThumbs(Context context)
 : _context(context) {
 }
@@ -509,10 +529,41 @@ auto GroupThumbs::createThumb(Key key)
 			}
 		}
 		return createThumb(key, ImagePtr());
+	} else if (const auto collageKey = base::get_if<CollageKey>(&key)) {
+		if (const auto itemId = base::get_if<FullMsgId>(&_context)) {
+			if (const auto item = App::histItemById(*itemId)) {
+				if (const auto media = item->media()) {
+					if (const auto page = media->webpage()) {
+						return createThumb(
+							key,
+							page->collage,
+							collageKey->index);
+					}
+				}
+			}
+		}
+		return createThumb(key, ImagePtr());
 	}
 	Unexpected("Value of Key in GroupThumbs::createThumb()");
 }
 
+auto GroupThumbs::createThumb(
+	Key key,
+	const WebPageCollage &collage,
+	int index)
+-> std::unique_ptr<Thumb> {
+	if (index < 0 || index >= collage.items.size()) {
+		return createThumb(key, ImagePtr());
+	}
+	const auto &item = collage.items[index];
+	if (const auto photo = base::get_if<PhotoData*>(&item)) {
+		return createThumb(key, (*photo)->thumb);
+	} else if (const auto document = base::get_if<DocumentData*>(&item)) {
+		return createThumb(key, (*document)->thumb);
+	}
+	return createThumb(key, ImagePtr());
+}
+
 auto GroupThumbs::createThumb(Key key, ImagePtr image)
 -> std::unique_ptr<Thumb> {
 	const auto weak = base::make_weak(this);
@@ -555,6 +606,14 @@ void GroupThumbs::Refresh(
 	RefreshFromSlice(instance, slice, index, availableWidth);
 }
 
+void GroupThumbs::Refresh(
+		std::unique_ptr<GroupThumbs> &instance,
+		const CollageSlice &slice,
+		int index,
+		int availableWidth) {
+	RefreshFromSlice(instance, slice, index, availableWidth);
+}
+
 void GroupThumbs::clear() {
 	if (_items.empty()) {
 		return;
