@@ -379,8 +379,8 @@ MainWindow::MainWindow()
 	auto forceOpenGL = std::make_unique<QOpenGLWidget>(this);
 #endif // !OS_MAC_OLD
 
-	trayImg = st::macTrayIcon.instance(QColor(0, 0, 0, 180), dbisOne);
-	trayImgSel = st::macTrayIcon.instance(QColor(255, 255, 255), dbisOne);
+	trayImg = st::macTrayIcon.instance(QColor(0, 0, 0, 180), 100);
+	trayImgSel = st::macTrayIcon.instance(QColor(255, 255, 255), 100);
 
 	_hideAfterFullScreenTimer.setCallback([this] { hideAndDeactivate(); });
 
@@ -522,21 +522,24 @@ void MainWindow::unreadCounterChangedHook() {
 }
 
 void MainWindow::updateIconCounters() {
-	auto counter = App::histories().unreadBadge();
+	const auto counter = Messenger::Instance().unreadBadge();
+	const auto muted = Messenger::Instance().unreadBadgeMuted();
 
-	QString cnt = (counter < 1000) ? QString("%1").arg(counter) : QString("..%1").arg(counter % 100, 2, 10, QChar('0'));
-	_private->setWindowBadge(counter ? cnt : QString());
+	const auto string = !counter
+		? QString()
+		: (counter < 1000)
+		? QString("%1").arg(counter)
+		: QString("..%1").arg(counter % 100, 2, 10, QChar('0'));
+	_private->setWindowBadge(string);
 
 	if (trayIcon) {
-		bool muted = App::histories().unreadOnlyMuted();
 		bool dm = objc_darkMode();
-
 		auto &bg = (muted ? st::trayCounterBgMute : st::trayCounterBg);
 		QIcon icon;
 		QImage img(psTrayIcon(dm)), imgsel(psTrayIcon(true));
 		img.detach();
 		imgsel.detach();
-		int32 size = cRetina() ? 44 : 22;
+		int32 size = 22 * cIntRetinaFactor();
 		_placeCounter(img, size, counter, bg, (dm && muted) ? st::trayCounterFgMacInvert : st::trayCounterFg);
 		_placeCounter(imgsel, size, counter, st::trayCounterBgMacInvert, st::trayCounterFgMacInvert);
 		icon.addPixmap(App::pixmapFromImageInPlace(std::move(img)));
@@ -699,6 +702,7 @@ void MainWindow::updateGlobalMenuHook() {
 	const auto logged = AuthSession::Exists();
 	const auto locked = !Messenger::Instance().locked();
 	const auto inactive = !logged || locked;
+	const auto support = logged && Auth().supportMode();
 	_forceDisabled(psLogout, !logged && !locked);
 	_forceDisabled(psUndo, !canUndo);
 	_forceDisabled(psRedo, !canRedo);
@@ -707,10 +711,10 @@ void MainWindow::updateGlobalMenuHook() {
 	_forceDisabled(psPaste, !canPaste);
 	_forceDisabled(psDelete, !canDelete);
 	_forceDisabled(psSelectAll, !canSelectAll);
-	_forceDisabled(psContacts, inactive);
+	_forceDisabled(psContacts, inactive || support);
 	_forceDisabled(psAddContact, inactive);
-	_forceDisabled(psNewGroup, inactive);
-	_forceDisabled(psNewChannel, inactive);
+	_forceDisabled(psNewGroup, inactive || support);
+	_forceDisabled(psNewChannel, inactive || support);
 	_forceDisabled(psShowTelegram, App::wnd()->isActive());
 }
 
