@@ -26,6 +26,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include <IOKit/hidsystem/ev_keymap.h>
 #include <SPMediaKeyTap.h>
 #include <mach-o/dyld.h>
+#include <AVFoundation/AVFoundation.h>
 
 namespace {
 
@@ -205,10 +206,6 @@ QString psAppDataPath() {
 	return objc_appDataPath();
 }
 
-QString psDownloadPath() {
-	return objc_downloadPath();
-}
-
 void psDoCleanup() {
 	try {
 		psAutoStart(false, true);
@@ -283,6 +280,58 @@ void RegisterCustomScheme() {
 #endif // !TDESKTOP_DISABLE_REGISTER_CUSTOM_SCHEME
 }
 
+// I do check for availability, just not in the exact way clang is content with
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wunguarded-availability"
+PermissionStatus GetPermissionStatus(PermissionType type) {
+#ifndef OS_MAC_OLD
+	switch (type) {
+		case PermissionType::Microphone:
+			if([AVCaptureDevice respondsToSelector: @selector(authorizationStatusForMediaType:)]) { // Available starting with 10.14
+				switch([AVCaptureDevice authorizationStatusForMediaType:AVMediaTypeAudio]) {
+					case AVAuthorizationStatusNotDetermined:
+						return PermissionStatus::CanRequest;
+					case AVAuthorizationStatusAuthorized:
+						return PermissionStatus::Granted;
+					case AVAuthorizationStatusDenied:
+					case AVAuthorizationStatusRestricted:
+						return PermissionStatus::Denied;
+				}
+			}
+			break;
+	}
+#endif // OS_MAC_OLD
+	return PermissionStatus::Granted;
+}
+
+void RequestPermission(PermissionType type, Fn<void(PermissionStatus)> resultCallback) {
+#ifndef OS_MAC_OLD
+	switch (type) {
+		case PermissionType::Microphone:
+			if ([AVCaptureDevice respondsToSelector: @selector(requestAccessForMediaType:completionHandler:)]) { // Available starting with 10.14
+				[AVCaptureDevice requestAccessForMediaType:AVMediaTypeAudio completionHandler:^(BOOL granted) {
+					crl::on_main([=] {
+						resultCallback(granted ? PermissionStatus::Granted : PermissionStatus::Denied);
+					});
+				}];
+			}
+			break;
+	}
+#endif // OS_MAC_OLD
+	resultCallback(PermissionStatus::Granted);
+}
+#pragma clang diagnostic pop // -Wunguarded-availability
+
+void OpenSystemSettingsForPermission(PermissionType type) {
+#ifndef OS_MAC_OLD
+	switch (type) {
+		case PermissionType::Microphone:
+			[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"x-apple.systempreferences:com.apple.preference.security?Privacy_Microphone"]];
+			break;
+	}
+#endif // OS_MAC_OLD
+}
+
 } // namespace Platform
 
 void psNewVersion() {
