@@ -12,14 +12,28 @@ namespace Platform {
 void start();
 void finish();
 
+enum class PermissionStatus {
+	Granted,
+	CanRequest,
+	Denied,
+};
+
+enum class PermissionType {
+	Microphone,
+};
+
 void SetWatchingMediaKeys(bool watching);
 bool IsApplicationActive();
+void SetApplicationIcon(const QIcon &icon);
 bool TranslucentWindowsSupported(QPoint globalPosition);
 void StartTranslucentPaint(QPainter &p, QPaintEvent *e);
 void InitOnTopPanel(QWidget *panel);
 void DeInitOnTopPanel(QWidget *panel);
 void ReInitOnTopPanel(QWidget *panel);
 void RegisterCustomScheme();
+PermissionStatus GetPermissionStatus(PermissionType type);
+void RequestPermission(PermissionType type, Fn<void(PermissionStatus)> resultCallback);
+void OpenSystemSettingsForPermission(PermissionType type);
 
 QString SystemLanguage();
 QString SystemCountry();
