@@ -43,7 +43,7 @@ void SetupConnectionType(not_null<Ui::VerticalLayout*> container) {
 #ifndef TDESKTOP_DISABLE_NETWORK_PROXY
 	const auto connectionType = [] {
 		const auto transport = MTP::dctransport();
-		if (!Global::UseProxy()) {
+		if (Global::ProxySettings() != ProxyData::Settings::Enabled) {
 			return transport.isEmpty()
 				? lang(lng_connection_auto_connecting)
 				: lng_connection_auto(lt_transport, transport);
@@ -77,12 +77,10 @@ void SetupUpdate(not_null<Ui::VerticalLayout*> container) {
 		return;
 	}
 
-	const auto texts = Ui::AttachAsChild(
-		container,
-		rpl::event_stream<QString>());
-	const auto downloading = Ui::AttachAsChild(
-		container,
-		rpl::event_stream<bool>());
+	const auto texts = Ui::CreateChild<rpl::event_stream<QString>>(
+		container.get());
+	const auto downloading = Ui::CreateChild<rpl::event_stream<bool>>(
+		container.get());
 	const auto version = lng_settings_current_version(
 		lt_version,
 		currentVersionText());
@@ -298,8 +296,7 @@ void SetupTrayContent(not_null<Ui::VerticalLayout*> container) {
 		Local::writeSettings();
 	};
 
-	base::ObservableViewer(
-		tray->checkedChanged
+	tray->checkedChanges(
 	) | rpl::filter([=](bool checked) {
 		return (checked != trayEnabled());
 	}) | rpl::start_with_next([=](bool checked) {
@@ -311,8 +308,7 @@ void SetupTrayContent(not_null<Ui::VerticalLayout*> container) {
 	}, tray->lifetime());
 
 	if (taskbar) {
-		base::ObservableViewer(
-			taskbar->checkedChanged
+		taskbar->checkedChanges(
 		) | rpl::filter([=](bool checked) {
 			return (checked != taskbarEnabled());
 		}) | rpl::start_with_next([=](bool checked) {
@@ -340,8 +336,7 @@ void SetupTrayContent(not_null<Ui::VerticalLayout*> container) {
 			lng_settings_add_sendto,
 			cSendToMenu());
 
-		base::ObservableViewer(
-			autostart->checkedChanged
+		autostart->checkedChanges(
 		) | rpl::filter([](bool checked) {
 			return (checked != cAutoStart());
 		}) | rpl::start_with_next([=](bool checked) {
@@ -356,13 +351,8 @@ void SetupTrayContent(not_null<Ui::VerticalLayout*> container) {
 			}
 		}, autostart->lifetime());
 
-		minimized->toggleOn(rpl::single(
-			autostart->checked()
-		) | rpl::then(base::ObservableViewer(
-			autostart->checkedChanged
-		)));
-		base::ObservableViewer(
-			minimized->entity()->checkedChanged
+		minimized->toggleOn(autostart->checkedValue());
+		minimized->entity()->checkedChanges(
 		) | rpl::filter([=](bool checked) {
 			return (checked != minimizedToggled());
 		}) | rpl::start_with_next([=](bool checked) {
@@ -382,8 +372,7 @@ void SetupTrayContent(not_null<Ui::VerticalLayout*> container) {
 			minimized->entity()->setChecked(minimizedToggled());
 		}, minimized->lifetime());
 
-		base::ObservableViewer(
-			sendto->checkedChanged
+		sendto->checkedChanges(
 		) | rpl::filter([](bool checked) {
 			return (checked != cSendToMenu());
 		}) | rpl::start_with_next([](bool checked) {
