@@ -15,9 +15,12 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "boxes/local_storage_box.h"
 #include "ui/wrap/vertical_layout.h"
 #include "ui/wrap/slide_wrap.h"
+#include "ui/widgets/input_fields.h"
 #include "ui/widgets/checkbox.h"
 #include "ui/widgets/labels.h"
 #include "ui/effects/radial_animation.h"
+#include "ui/toast/toast.h"
+#include "ui/image/image.h"
 #include "lang/lang_keys.h"
 #include "window/themes/window_theme_editor.h"
 #include "window/themes/window_theme.h"
@@ -25,6 +28,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "storage/localstorage.h"
 #include "core/file_utilities.h"
 #include "data/data_session.h"
+#include "support/support_common.h"
+#include "support/support_templates.h"
 #include "auth_session.h"
 #include "mainwidget.h"
 #include "styles/style_settings.h"
@@ -436,11 +441,10 @@ void SetupStickersEmoji(not_null<Ui::VerticalLayout*> container) {
 			st::settingsCheckbox);
 	};
 	const auto add = [&](LangKey label, bool checked, auto &&handle) {
-		base::ObservableViewer(
-			inner->add(
-				checkbox(label, checked),
-				st::settingsCheckboxPadding
-			)->checkedChanged
+		inner->add(
+			checkbox(label, checked),
+			st::settingsCheckboxPadding
+		)->checkedChanges(
 		) | rpl::start_with_next(
 			std::move(handle),
 			inner->lifetime());
@@ -491,14 +495,9 @@ void SetupMessages(not_null<Ui::VerticalLayout*> container) {
 
 	AddSkip(container, st::settingsSendTypeSkip);
 
-	enum class SendByType {
-		Enter,
-		CtrlEnter,
-	};
+	using SendByType = Ui::InputSubmitSettings;
 
 	const auto skip = st::settingsSendTypeSkip;
-	const auto group = std::make_shared<Ui::RadioenumGroup<SendByType>>(
-		cCtrlEnter() ? SendByType::CtrlEnter : SendByType::Enter);
 	auto wrap = object_ptr<Ui::VerticalLayout>(container);
 	const auto inner = wrap.data();
 	container->add(
@@ -507,10 +506,9 @@ void SetupMessages(not_null<Ui::VerticalLayout*> container) {
 			std::move(wrap),
 			QMargins(0, skip, 0, skip)));
 
-	const auto add = [&](
-		SendByType value,
-		LangKey key,
-		style::margins padding) {
+	const auto group = std::make_shared<Ui::RadioenumGroup<SendByType>>(
+		Auth().settings().sendSubmitWay());
+	const auto add = [&](SendByType value, LangKey key) {
 		inner->add(
 			object_ptr<Ui::Radioenum<SendByType>>(
 				inner,
@@ -522,18 +520,15 @@ void SetupMessages(not_null<Ui::VerticalLayout*> container) {
 	};
 	const auto small = st::settingsSendTypePadding;
 	const auto top = skip;
-	add(
-		SendByType::Enter,
-		lng_settings_send_enter,
-		{ small.left(), small.top() + top, small.right(), small.bottom() });
+	add(SendByType::Enter, lng_settings_send_enter);
 	add(
 		SendByType::CtrlEnter,
 		((cPlatform() == dbipMac || cPlatform() == dbipMacOld)
 			? lng_settings_send_cmdenter
-			: lng_settings_send_ctrlenter),
-		{ small.left(), small.top(), small.right(), small.bottom() + top });
+			: lng_settings_send_ctrlenter));
+
 	group->setChangedCallback([](SendByType value) {
-		cSetCtrlEnter(value == SendByType::CtrlEnter);
+		Auth().settings().setSendSubmitWay(value);
 		if (App::main()) {
 			App::main()->ctrlEnterSubmitUpdated();
 		}
@@ -582,9 +577,7 @@ void SetupDataStorage(not_null<Ui::VerticalLayout*> container) {
 	)->toggleOn(rpl::single(Global::AskDownloadPath()));
 
 #ifndef OS_WIN_STORE
-	const auto showpath = Ui::AttachAsChild(
-		ask,
-		rpl::event_stream<bool>());
+	const auto showpath = Ui::CreateChild<rpl::event_stream<bool>>(ask);
 	const auto path = container->add(
 		object_ptr<Ui::SlideWrap<Button>>(
 			container,
@@ -676,8 +669,7 @@ void SetupChatBackground(not_null<Ui::VerticalLayout*> container) {
 				st::settingsCheckbox),
 			st::settingsSendTypePadding));
 
-	base::ObservableViewer(
-		tile->checkedChanged
+	tile->checkedChanges(
 	) | rpl::start_with_next([](bool checked) {
 		Window::Theme::Background()->setTile(checked);
 	}, tile->lifetime());
@@ -701,8 +693,7 @@ void SetupChatBackground(not_null<Ui::VerticalLayout*> container) {
 		return (Global::AdaptiveChatLayout() == Adaptive::ChatLayout::Wide);
 	}));
 
-	base::ObservableViewer(
-		adaptive->entity()->checkedChanged
+	adaptive->entity()->checkedChanges(
 	) | rpl::start_with_next([](bool checked) {
 		Global::SetAdaptiveForWide(checked);
 		Adaptive::Changed().notify();
@@ -710,29 +701,6 @@ void SetupChatBackground(not_null<Ui::VerticalLayout*> container) {
 	}, adaptive->lifetime());
 }
 
-void SetupUseDefaultTheme(not_null<Ui::VerticalLayout*> container) {
-	using Update = const Window::Theme::BackgroundUpdate;
-	container->add(
-		object_ptr<Ui::SlideWrap<Button>>(
-			container,
-			object_ptr<Button>(
-				container,
-				Lang::Viewer(lng_settings_bg_use_default),
-				st::settingsButton))
-	)->toggleOn(rpl::single(
-		Window::Theme::SuggestThemeReset()
-	) | rpl::then(base::ObservableViewer(
-		*Window::Theme::Background()
-	) | rpl::filter([](const Update &update) {
-		return (update.type == Update::Type::ApplyingTheme
-			|| update.type == Update::Type::New);
-	}) | rpl::map([] {
-		return Window::Theme::SuggestThemeReset();
-	})))->entity()->addClickHandler([] {
-		Window::Theme::ApplyDefault();
-	});
-}
-
 void SetupDefaultThemes(not_null<Ui::VerticalLayout*> container) {
 	using Type = DefaultTheme::Type;
 	using Scheme = DefaultTheme::Scheme;
@@ -932,11 +900,113 @@ void SetupThemeOptions(not_null<Ui::VerticalLayout*> container) {
 		container,
 		[] { Window::Theme::Editor::Start(); }));
 
-	//SetupUseDefaultTheme(container);
-
 	AddSkip(container);
 }
 
+void SetupSupportSwitchSettings(not_null<Ui::VerticalLayout*> container) {
+	using SwitchType = Support::SwitchSettings;
+	const auto group = std::make_shared<Ui::RadioenumGroup<SwitchType>>(
+		Auth().settings().supportSwitch());
+	const auto add = [&](SwitchType value, const QString &label) {
+		container->add(
+			object_ptr<Ui::Radioenum<SwitchType>>(
+				container,
+				group,
+				value,
+				label,
+				st::settingsSendType),
+			st::settingsSendTypePadding);
+	};
+	add(SwitchType::None, "Just send the reply");
+	add(SwitchType::Next, "Send and switch to next");
+	add(SwitchType::Previous, "Send and switch to previous");
+	group->setChangedCallback([](SwitchType value) {
+		Auth().settings().setSupportSwitch(value);
+		Local::writeUserSettings();
+	});
+}
+
+void SetupSupportChatsLimitSlice(not_null<Ui::VerticalLayout*> container) {
+	constexpr auto kDayDuration = 24 * 60 * 60;
+	struct Option {
+		int days = 0;
+		QString label;
+	};
+	const auto options = std::vector<Option>{
+		{ 1, "1 day" },
+		{ 7, "1 week" },
+		{ 30, "1 month" },
+		{ 365, "1 year" },
+		{ 0, "All of them" },
+	};
+	const auto current = Auth().settings().supportChatsTimeSlice();
+	const auto days = current / kDayDuration;
+	const auto best = ranges::min_element(
+		options,
+		std::less<>(),
+		[&](const Option &option) { return std::abs(option.days - days); });
+
+	const auto group = std::make_shared<Ui::RadiobuttonGroup>(best->days);
+	for (const auto &option : options) {
+		container->add(
+			object_ptr<Ui::Radiobutton>(
+				container,
+				group,
+				option.days,
+				option.label,
+				st::settingsSendType),
+			st::settingsSendTypePadding);
+	}
+	group->setChangedCallback([=](int days) {
+		Auth().settings().setSupportChatsTimeSlice(days * kDayDuration);
+		Local::writeUserSettings();
+	});
+}
+
+void SetupSupport(not_null<Ui::VerticalLayout*> container) {
+	AddSkip(container);
+
+	AddSubsectionTitle(container, rpl::single(qsl("Support settings")));
+
+	AddSkip(container, st::settingsSendTypeSkip);
+
+	const auto skip = st::settingsSendTypeSkip;
+	auto wrap = object_ptr<Ui::VerticalLayout>(container);
+	const auto inner = wrap.data();
+	container->add(
+		object_ptr<Ui::OverrideMargins>(
+			container,
+			std::move(wrap),
+			QMargins(0, skip, 0, skip)));
+
+	SetupSupportSwitchSettings(inner);
+
+	AddSkip(inner, st::settingsCheckboxesSkip);
+
+	inner->add(
+		object_ptr<Ui::Checkbox>(
+			inner,
+			"Enable templates autocomplete",
+			Auth().settings().supportTemplatesAutocomplete(),
+			st::settingsCheckbox),
+		st::settingsSendTypePadding
+	)->checkedChanges(
+	) | rpl::start_with_next([=](bool checked) {
+		Auth().settings().setSupportTemplatesAutocomplete(checked);
+		Local::writeUserSettings();
+	}, inner->lifetime());
+
+	AddSkip(inner, st::settingsCheckboxesSkip);
+
+	AddSubsectionTitle(inner, rpl::single(qsl("Load chats for a period")));
+
+	SetupSupportChatsLimitSlice(inner);
+
+	AddSkip(inner, st::settingsCheckboxesSkip);
+
+	AddSkip(inner);
+}
+
 Chat::Chat(QWidget *parent, not_null<UserData*> self)
 : Section(parent)
 , _self(self) {
