@@ -91,7 +91,19 @@ void Start() {
 			LOG(("Fonts Info: Using Segoe UI Semibold instead of Open Sans Semibold."));
 		}
 	}
-#endif // Q_OS_WIN
+	// Disable default fallbacks to Segoe UI, see:
+	// https://github.com/telegramdesktop/tdesktop/issues/5368
+	//
+	//QFont::insertSubstitution(qsl("Open Sans"), qsl("Segoe UI"));
+	//QFont::insertSubstitution(qsl("Open Sans Semibold"), qsl("Segoe UI Semibold"));
+#elif defined Q_OS_MAC // Q_OS_WIN
+	auto list = QStringList();
+	list.append(qsl(".SF NS Text"));
+	list.append(qsl("Helvetica Neue"));
+	list.append(qsl("Lucida Grande"));
+	QFont::insertSubstitutions(qsl("Open Sans"), list);
+	QFont::insertSubstitutions(qsl("Open Sans Semibold"), list);
+#endif // Q_OS_WIN || Q_OS_MAC
 }
 
 QString GetOverride(const QString &familyName) {
@@ -221,6 +233,11 @@ void ForceFullRepaint(not_null<QWidget*> widget) {
 	refresher->show();
 }
 
+void PostponeCall(FnMut<void()> &&callable) {
+	const auto app = static_cast<Application*>(qApp);
+	app->postponeCall(std::move(callable));
+}
+
 } // namespace Ui
 
 void sendSynteticMouseEvent(QWidget *widget, QEvent::Type type, Qt::MouseButton button, const QPoint &globalPoint) {
